"""Original code: https://github.com/halfak/deep_merge"""
from __future__ import annotations

from typing import Any, TypeVar, Callable, Protocol

from typing_extensions import TypeAlias  # noqa[TC002]

from checkov.common.util.data_structures_utils import pickle_deepcopy

_T = TypeVar("_T")
_MergeDict = TypeVar("_MergeDict", bound="dict[Any, Any]")
_OverwriteFunc: TypeAlias = "Callable[..., Any]"


class _MergeDictsFunc(Protocol):
    def __call__(
        self,
        d1: _MergeDict,
        d2: _MergeDict,
        *,
        merge_lists: _OverwriteFunc,
        merge_ints: _OverwriteFunc,
        merge_floats: _OverwriteFunc,
        merge_strings: _OverwriteFunc,
        merge_other: _OverwriteFunc,
    ) -> _MergeDict:
        ...


def overwrite(v1: _T, v2: _T, **kwargs: Any) -> _T:
    """
    Completely overwrites one value with another.
    """
    return pickle_deepcopy(v2)


def merge_dicts(
    d1: _MergeDict,
    d2: _MergeDict,
    merge_lists: _OverwriteFunc = overwrite,
    merge_ints: _OverwriteFunc = overwrite,
    merge_floats: _OverwriteFunc = overwrite,
    merge_strings: _OverwriteFunc = overwrite,
    merge_other: _OverwriteFunc = overwrite,
) -> _MergeDict:
    """
    Recursively merges values from d2 into d1.
    """
    kwargs = {
        "merge_lists": merge_lists,
        "merge_ints": merge_ints,
        "merge_floats": merge_floats,
        "merge_strings": merge_strings,
        "merge_other": merge_other,
    }
    for key in d2:
        if key in d1:
            if isinstance(d1[key], dict) and isinstance(d2[key], dict):
                d1[key] = merge_dicts(d1[key], d2[key], **kwargs)
            elif isinstance(d1[key], list) and isinstance(d2[key], list):
                d1[key] = merge_lists(d1[key], d2[key], **kwargs)
            elif isinstance(d1[key], int) and isinstance(d2[key], int):
                d1[key] = merge_ints(d1[key], d2[key], **kwargs)
            elif isinstance(d1[key], float) and isinstance(d2[key], float):
                d1[key] = merge_ints(d1[key], d2[key], **kwargs)
            elif isinstance(d1[key], str) and isinstance(d2[key], str):
                d1[key] = merge_strings(d1[key], d2[key], **kwargs)
            else:
                d1[key] = merge_other(d1[key], d2[key], **kwargs)
        else:
            d1[key] = overwrite(None, d2[key])

    return d1


def pickle_deep_merge(
    *dicts: _MergeDict,
    merge_dicts: _MergeDictsFunc = merge_dicts,
    merge_lists: _OverwriteFunc = overwrite,
    merge_ints: _OverwriteFunc = overwrite,
    merge_floats: _OverwriteFunc = overwrite,
    merge_strings: _OverwriteFunc = overwrite,
    merge_other: _OverwriteFunc = overwrite,
) -> _MergeDict:
    """
    Recursively merges dictionaries and the datastructures they contain.

    :Parameters:
        *dicts : `dict`
            Dictionaries to be merged.  Items that appear last will take higher
            precedence when merging.
        merge_dicts : `func`
            The function to apply when merging dictionaries.
        merge_lists : `func`
            The function to apply when merging lists.
        merge_ints : `func`
            The function to apply when merging integers.
        merge_floats : `func`
            The function to apply when merging floats.
        merge_strings : `func`
            The function to apply when merging strings.
        merge_other : `func`
            The function to apply when merging other types or types that do not
            match.
    """
    for param in dicts:
        if not isinstance(param, dict):
            raise TypeError("{0} is not a dict".format(param))

    d = dicts[0]
    for d_update in dicts[1:]:
        d = merge_dicts(
            d,
            d_update,
            merge_lists=merge_lists,
            merge_ints=merge_ints,
            merge_floats=merge_floats,
            merge_strings=merge_strings,
            merge_other=merge_other,
        )

    return d



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_int


class RequireTwoApprovalsToMerge(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure at least two approving reviews are required to merge a GitLab MR"
        id = "CKV_GLB_1"
        supported_resources = ["gitlab_project"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        approvals = conf.get("approvals_before_merge")
        if approvals and isinstance(approvals, list):
            num_approvals = force_int(approvals[0])
            if num_approvals and num_approvals >= 2:
                return CheckResult.PASSED
        self.evaluated_keys = ["approvals_before_merge"]
        return CheckResult.FAILED


check = RequireTwoApprovalsToMerge()



from typing import Any
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class AppServiceJavaVersion(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that 'Java version' is the latest, if used to run the web app"
        id = "CKV_AZURE_83"
        supported_resources = ('Microsoft.Web/sites',)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.UNKNOWN)

    def get_inspected_key(self) -> str:
        return "siteConfig/javaVersion"

    def get_expected_value(self) -> Any:
        return '17'


check = AppServiceJavaVersion()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class CodecommitApprovalsRulesRequireMin2(BaseResourceCheck):

    def __init__(self):
        """
        See https://docs.aws.amazon.com/codecommit/latest/userguide/approval-rule-templates.html
        """
        name = "Ensure CodeCommit branch changes have at least 2 approvals"
        id = "CKV_AWS_257"
        supported_resources = ['aws_codecommit_approval_rule_template']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if conf.get("content") and isinstance(conf.get("content"), list):
            content = conf.get("content")[0]
            if not isinstance(content, dict):
                return CheckResult.UNKNOWN
            if content.get("Statements") and isinstance(content.get("Statements"), list):
                statement = content.get("Statements")[0]
                if isinstance(statement.get('NumberOfApprovalsNeeded'), int) and statement.get('NumberOfApprovalsNeeded') >= 2:
                    return CheckResult.PASSED
                self.evaluated_keys = ["content/Statements/NumberOfApprovalsNeeded"]
        return CheckResult.FAILED


check = CodecommitApprovalsRulesRequireMin2()



from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class CognitiveServicesEnableLocalAuth(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Cognitive Services accounts enable local authentication"
        id = "CKV_AZURE_236"
        supported_resources = ('Microsoft.CognitiveServices/accounts', )
        categories = (CheckCategories.NETWORKING, )
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def get_inspected_key(self) -> str:
        return 'properties/disableLocalAuth'

    def get_expected_value(self) -> Any:
        return True


check = CognitiveServicesEnableLocalAuth()



from checkov.terraform.checks.resource.oci.AbsSecurityListUnrestrictedIngress import AbsSecurityListUnrestrictedIngress


class SecurityListUnrestrictedIngress3389(AbsSecurityListUnrestrictedIngress):
    def __init__(self):
        super().__init__(check_id="CKV_OCI_20", port=3389, is_exposed_by_default=False)


check = SecurityListUnrestrictedIngress3389()



from __future__ import annotations

import logging
from collections.abc import Iterable
from typing import TYPE_CHECKING, Any

import yaml

import checkov.common.parsers.yaml.loader as loader
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.graph.graph_builder.consts import GraphSource
from checkov.common.output.report import Report
from checkov.github_actions.checks.registry import registry
from checkov.github_actions.graph_builder.local_graph import GitHubActionsLocalGraph
from checkov.github_actions.utils import is_schema_valid, is_workflow_file
from checkov.runner_filter import RunnerFilter
from checkov.yaml_doc.runner import Runner as YamlRunner

if TYPE_CHECKING:
    from checkov.common.checks.base_check_registry import BaseCheckRegistry
    from checkov.common.typing import LibraryGraphConnector
    from checkov.common.runners.graph_builder.local_graph import ObjectLocalGraph
    from checkov.common.runners.graph_manager import ObjectGraphManager


class Runner(YamlRunner):
    check_type = CheckType.GITHUB_ACTIONS  # noqa: CCE003  # a static attribute

    def __init__(
        self,
        db_connector: LibraryGraphConnector | None = None,
        source: str = GraphSource.GITHUB_ACTIONS,
        graph_class: type[ObjectLocalGraph] = GitHubActionsLocalGraph,
        graph_manager: ObjectGraphManager | None = None,
        external_registries: dict[str, Any] | None = None
    ) -> None:
        super().__init__(
            db_connector=db_connector,
            source=source,
            graph_class=graph_class,
            graph_manager=graph_manager,
        )

    def require_external_checks(self) -> bool:
        return False

    def import_registry(self) -> BaseCheckRegistry:
        return registry

    @staticmethod
    def _parse_file(f: str, file_content: str | None = None) -> \
            tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
        if is_workflow_file(f):
            entity_schema: tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None = YamlRunner._parse_file(f)
            if not entity_schema:
                # Indicates that there was an exception/error while trying to read the file,
                # hence no need to check the schema validity.
                return None

            if not file_content:
                with open(f, 'r') as f_obj:
                    try:
                        file_content = f_obj.read()
                    except Exception as e:
                        logging.warning(f'Fail to read file {f}. error: {e}')
                        return None

            if all(map(is_schema_valid, yaml.load_all(file_content, Loader=loader.SafeLineLoaderGhaSchema))):  # nosec
                return entity_schema
        return None

    def included_paths(self) -> Iterable[str]:
        return [".github"]

    def get_resource(self, file_path: str, key: str, supported_entities: Iterable[str],
                     start_line: int = -1, end_line: int = -1, graph_resource: bool = False) -> str:
        """
        supported resources for GHA:
            jobs
            jobs.*.steps[]
            permissions
            on

        """
        if len(list(supported_entities)) > 1:
            logging.debug("order of entities might cause extracting the wrong key for resource_id")
        new_key = key
        definition = self.definitions.get(file_path, {})
        if not definition or not isinstance(definition, dict):
            return new_key
        if 'on' in supported_entities:
            workflow_name = definition.get('name', "")
            new_key = f"on({workflow_name})" if workflow_name else "on"
        elif 'jobs' in supported_entities or graph_resource and 'steps' in supported_entities:
            job_name = self.resolve_sub_name(definition, start_line, end_line, tag='jobs')
            new_key = f"jobs({job_name})" if job_name else "jobs"

            if (graph_resource and supported_entities == 'steps') or \
                    (not graph_resource and 'jobs.*.steps[]' in supported_entities and key.split('.')[1] == '*'):
                step_name = self.resolve_step_name(definition['jobs'].get(job_name), start_line, end_line)
                new_key = f'jobs({job_name}).steps{step_name}'
        elif 'permissions' in supported_entities:
            new_key = 'permissions'
        return new_key

    def run(
            self,
            root_folder: str | None = None,
            external_checks_dir: list[str] | None = None,
            files: list[str] | None = None,
            runner_filter: RunnerFilter | None = None,
            collect_skip_comments: bool = True,
    ) -> Report | list[Report]:
        runner_filter = runner_filter or RunnerFilter()
        report = super().run(root_folder=root_folder, external_checks_dir=external_checks_dir,
                             files=files, runner_filter=runner_filter, collect_skip_comments=collect_skip_comments)
        return report

    def populate_metadata_dict(self) -> None:
        if isinstance(self.definitions, dict):
            # populate gha metadata dict
            for key, definition in self.definitions.items():
                if isinstance(definition, dict):
                    workflow_name = definition.get('name', '')
                    triggers = self._get_triggers(definition)
                    jobs = self._get_jobs(definition)
                    self.map_file_path_to_gha_metadata_dict[key] = {"triggers": triggers,
                                                                    "workflow_name": workflow_name, "jobs": jobs}



from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class SagemakerDataQualityJobDefinitionTrafficEncryption(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Amazon Sagemaker Data Quality Job encrypts all communications between instances used for monitoring jobs"
        id = "CKV_AWS_369"
        supported_resources = ["AWS::SageMaker::DataQualityJobDefinition"]
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'Properties/NetworkConfig/EnableInterContainerTrafficEncryption'


check = SagemakerDataQualityJobDefinitionTrafficEncryption()



import json
import os
from typing import List, Dict, Any
import yaml


current_dir = os.path.dirname(os.path.realpath(__file__))


def load_failed_checks_from_file(lang: str) -> Dict[str, List[Dict[str, Any]]]:
    report_path = os.path.join(current_dir, '..', 'checkov_report_cdk.json')
    with open(report_path) as f:
        data = f.read()
        reports = json.loads(data)
        for report in reports:
            if report.get('check_type') == f'cdk_{lang}':
                assert report is not None
                results = report.get("results", {})
                failed_checks = results.get("failed_checks")
                skipped_checks = results.get("skipped_checks")
                results = {}
                for check in failed_checks:
                    check_id = check['check_id']
                    if not results.get(check_id):
                        results[check_id] = []
                    results[check_id].append(check)
                for check in skipped_checks:
                    check_id = check['check_id']
                    if not results.get(check_id):
                        results[check_id] = []
                    results[check_id].append(check)
                return results
    return {}


def is_policy_with_correct_check_id(check_id: str, language: str, policy_name: str) -> bool:
    path = os.path.join(current_dir, '..', 'checkov', 'cdk', 'checks', language, policy_name + ".yaml")
    with open(path, 'r') as file:
        data = yaml.safe_load(file)
    if 'metadata' in data and 'id' in data['metadata'] and data['metadata']['id'] == check_id:
        return True
    return False


def run_check(check_results: Dict[str, List[Dict[str, Any]]], check_id: str, policy_name: str, language: str) -> None:
    assert is_policy_with_correct_check_id(check_id, language, policy_name)
    results_for_check_id = check_results.get(check_id)
    assert results_for_check_id


def validate_report(report_path: str) -> None:
    with open(report_path) as f:
        data = f.read()
        report = json.loads(data)
        assert report is not None
        results = report.get("results")
        assert results is not None
        passed_checks = results.get("passed_checks")
        failed_checks = results.get("failed_checks")
        assert not passed_checks
        assert failed_checks is not None
        assert isinstance(failed_checks, list)
        assert len(failed_checks) > 0
        summary = report.get("summary")
        assert summary.get("passed") == 0
        assert summary.get("failed") > 0



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class KeyspacesTableUsesCMK(BaseResourceCheck):
    """
    Valid values for encryption_specification type:
    AWS_OWNED_KMS_KEY (default), CUSTOMER_MANAGED_KMS_KEY (requires kms_key_identifier:ARN)
    """
    def __init__(self):
        name = "Ensure Keyspaces Table uses CMK"
        id = "CKV_AWS_265"
        supported_resources = ['aws_keyspaces_table']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        if conf.get("encryption_specification") and isinstance(conf.get("encryption_specification"), list):
            encrypt = conf.get("encryption_specification")[0]
            if encrypt.get("kms_key_identifier") and isinstance(encrypt.get("kms_key_identifier"), list):
                if encrypt.get("type") == ["CUSTOMER_MANAGED_KMS_KEY"]:
                    return CheckResult.PASSED
                self.evaluated_keys = ["encryption_specification/[0]/type"]
            self.evaluated_keys = ["encryption_specification/[0]/kms_key_identifier"]
        self.evaluated_keys = ["encryption_specification"]
        return CheckResult.FAILED


check = KeyspacesTableUsesCMK()



from abc import abstractmethod
from collections.abc import Iterable
from typing import List, Any, Optional, Dict

from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck
from checkov.cloudformation.context_parser import ContextParser
from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckResult, CheckCategories


class BaseResourceNegativeValueCheck(BaseResourceCheck):
    def __init__(
        self,
        name: str,
        id: str,
        categories: "Iterable[CheckCategories]",
        supported_resources: "Iterable[str]",
        missing_block_result: CheckResult = CheckResult.FAILED,
    ) -> None:
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)
        self.missing_block_result = missing_block_result

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        excluded_key = self.get_excluded_key()
        if excluded_key is not None:
            path_elements = excluded_key.split("/")
            matches = ContextParser.search_deep_keys(path_elements[-1], conf, [])
            if len(matches) > 0:
                for match in matches:
                    if match[:-1] == path_elements:
                        if isinstance(match, list) and len(match) == 1:
                            match = match[0]
                        if self.check_excluded_condition(match):
                            return CheckResult.PASSED

        inspected_key = self.get_inspected_key()
        bad_values = self.get_forbidden_values()
        path_elements = inspected_key.split("/")
        matches = ContextParser.search_deep_keys(path_elements[-1], conf, [])
        if len(matches) > 0:
            for match in matches:
                if match[:-1] == path_elements:
                    if match[-1] in bad_values or ANY_VALUE in bad_values:
                        return CheckResult.FAILED

        return CheckResult.PASSED

    @abstractmethod
    def get_inspected_key(self) -> str:
        """
        :return: JSONPath syntax path of the checked attribute
        """
        raise NotImplementedError()

    @abstractmethod
    def get_forbidden_values(self) -> List[Any]:
        """
        Returns a list of vulnerable values for the inspected key, governed by provider best practices
        """
        raise NotImplementedError()

    def get_excluded_key(self) -> Optional[str]:
        """
        :return: JSONPath syntax path of the an attribute that provides exclusion condition for the inspected key
        """
        return None

    def check_excluded_condition(self, value: Any) -> bool:
        """
        :param:  value: value for  excluded_key
        :return: True if the value should exclude the check from failing if the inspected key has a bad value
        """
        return False

    def get_evaluated_keys(self) -> List[str]:
        return [self.get_inspected_key()]



from typing import Optional, Any, Dict, List, Union
from checkov.common.checks_infra.solvers.attribute_solvers.base_attribute_solver import BaseAttributeSolver
from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.util.type_forcers import force_int


class RangeIncludesAttributeSolver(BaseAttributeSolver):
    operator = Operators.RANGE_INCLUDES  # noqa: CCE003  # a static attribute

    def __init__(
            self, resource_types: List[str], attribute: Optional[str], value: Union[Any, List[Any]],
            is_jsonpath_check: bool = False
    ) -> None:
        # Convert value to a list if it's not already one to unify handling
        value = [force_int(v) if isinstance(v, (str, int)) else v for v in
                 (value if isinstance(value, list) else [value])]
        super().__init__(resource_types, attribute, value, is_jsonpath_check)

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        attr = vertex.get(attribute)  # type:ignore[arg-type]  # due to attribute can be None

        if attr is None:
            return False

        if isinstance(attr, list):
            return any(self._check_value(value, attr_val) for attr_val in attr for value in self.value)

        return any(self._check_value(value, attr) for value in self.value)

    def _check_value(self, value: Any, attr: Any) -> bool:
        # expects one of the following values:
        # - an actual int
        # - a string that parses to an int
        # - *
        # - a string range like '1000-2000'

        if attr == '*':
            return True

        if isinstance(attr, str) and attr.count("-") == 1:
            return self._check_range(value, attr)

        return bool(force_int(attr) == value)

    @staticmethod
    def _check_range(value: Any, range_str: str) -> bool:
        try:
            start, end = range_str.split("-")
            return bool(force_int(start) <= value <= force_int(end))
        except (TypeError, ValueError):
            # Occurs if there are not two entries or if one is not an int, in which case we just give up
            return False



from __future__ import annotations

import os
from logging import Logger, Filter, LogRecord

from checkov.common.util.type_forcers import convert_str_to_bool


class ResourceCodeFilter(Filter):
    """
    A custom logger filter designed to decide if we want to filter some logs from the default logger.
    Could be used to reduce logs size.
    First use case is to log without the actual code of resources, which takes a lot of the logs size.
    The default is to log everything in order to keep api the same.
    """
    CODE_TEMPLATES: list[str] = []

    def __init__(self, allow_code_logging: bool = True):
        super().__init__()
        self.allow_code_logging = allow_code_logging

    def filter(self, record: LogRecord) -> bool:
        if self.allow_code_logging:
            return True
        if hasattr(record, "mask"):
            # Allows filtering using `logging.info("<msg>", extra={"mask": True})`
            mask = record.mask
            if not isinstance(mask, bool):
                raise Exception(f"Expected to get `mask` as boolean for logging function, instead got: {mask} of type {type(mask)}")
            return not record.mask

        msg = record.msg
        return self._filter_based_on_msg(msg)

    def _filter_based_on_msg(self, msg: str) -> bool:
        for code_template in ResourceCodeFilter.CODE_TEMPLATES:
            if code_template in msg:
                return False
        return True


def add_resource_code_filter_to_logger(logger: Logger, allow_code_logging: bool | None = None) -> None:
    if allow_code_logging is None:
        allow_code_logging_res = convert_str_to_bool(os.environ.get("CHECKOV_ALLOW_CODE_LOGGING", True))
        if isinstance(allow_code_logging_res, bool):
            allow_code_logging = allow_code_logging_res
        else:
            raise Exception(f"Failed to get correct result for env variable - `CHECKOV_ALLOW_CODE_LOGGING`. "
                            f"Got {allow_code_logging_res}")

    resource_code_filter = ResourceCodeFilter(allow_code_logging=allow_code_logging)
    logger.addFilter(resource_code_filter)



from checkov.sast.checks_infra.base_registry import Registry
import pathlib
import os


checks_dir = pathlib.Path(__file__).parent.resolve().parent.resolve()
registry = Registry(os.path.join(checks_dir, 'checks'))



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class NetworkInterfaceEnableIPForwarding(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Azure File Sync disables public network access"
        id = "CKV_AZURE_64"
        supported_resources = ('Microsoft.StorageSync/storageSyncServices',)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.FAILED)

    def get_inspected_key(self) -> str:
        return 'properties/incomingTrafficPolicy'

    def get_expected_value(self) -> str:
        return 'AllowVirtualNetworksOnly'


check = NetworkInterfaceEnableIPForwarding()



from __future__ import annotations

import re
from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.module.base_module_check import BaseModuleCheck

COMMIT_ID_PATTERN = re.compile(r"\?(ref=)(?P<commit_id>([0-9a-f]{5,40}))")


class RevisionHash(BaseModuleCheck):
    def __init__(self) -> None:
        name = "Ensure Terraform module sources use a commit hash"
        id = "CKV_TF_1"
        categories = [CheckCategories.SUPPLY_CHAIN]
        super().__init__(name=name, id=id, categories=categories)

    def scan_module_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        source = conf.get("source")
        if source and isinstance(source, list):
            source_url = source[0]
            if source_url.startswith(("./", "../")):
                # local modules can't be pinned to a commit hash
                return CheckResult.UNKNOWN
            if "?ref" in source_url and re.search(COMMIT_ID_PATTERN, source_url):
                # do first a quick lookup, if '?ref' exists in the string before actually searching for the commit hash
                return CheckResult.PASSED

        return CheckResult.FAILED


check = RevisionHash()



from __future__ import annotations

from checkov.github.base_github_branch_security import BranchSecurity


class GithubBranchDismissStaleReviews(BranchSecurity):
    def __init__(self) -> None:
        name = "Ensure GitHub branch protection dismisses stale review on new commit"
        id = "CKV_GITHUB_11"
        super().__init__(name=name, id=id)

    def get_evaluated_keys(self) -> list[str]:
        return ["required_pull_request_reviews/dismiss_stale_reviews"]


check = GithubBranchDismissStaleReviews()



from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE


class KeyExpirationDate(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that the expiration date is set on all keys"
        id = "CKV_AZURE_40"
        supported_resources = ['Microsoft.KeyVault/vaults/keys']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'properties/rotationPolicy/attributes/expiryTime'

    def get_expected_value(self) -> str:
        return ANY_VALUE


check = KeyExpirationDate()



from __future__ import annotations
from typing import Any, Mapping

from checkov.common.typing import LibraryGraph
from checkov.kubernetes.image_referencer.base_provider import BaseKubernetesProvider
from checkov.common.images.graph.image_referencer_provider import _ExtractImagesCallableAlias
from checkov.common.graph.graph_builder.graph_components.attribute_names import CustomAttributes


class BaseKustomizeProvider(BaseKubernetesProvider):
    def __init__(self, graph_connector: LibraryGraph,
                 supported_resource_types: dict[str, _ExtractImagesCallableAlias] | Mapping[str, _ExtractImagesCallableAlias],
                 report_mutator_data: dict[str, dict[str, Any]]) -> None:
        super().__init__(
            graph_connector=graph_connector,
            supported_resource_types=supported_resource_types,
        )
        self.report_mutator_data = report_mutator_data

    def _get_resource_path(self, resource: dict[str, Any]) -> str:
        k8s_path = resource.get(CustomAttributes.FILE_PATH, "")
        dir_path = self.report_mutator_data.get('kustomizeFileMappings', {}).get(k8s_path, "")
        file_metadata = self.report_mutator_data.get('kustomizeMetadata', {}).get(dir_path, {})
        return str(file_metadata.get('filePath', ""))



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class DocDBEncryption(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure DocumentDB is encrypted at rest (default is unencrypted)"
        id = "CKV_AWS_74"
        supported_resources = ['aws_docdb_cluster']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "storage_encrypted"


check = DocDBEncryption()



from __future__ import annotations

import itertools
import logging
import re
from itertools import groupby
from typing import TYPE_CHECKING, Pattern, Any, Optional

from checkov.common.bridgecrew.check_type import CheckType

from checkov.common.bridgecrew.integration_features.base_integration_feature import BaseIntegrationFeature
from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import \
    integration as metadata_integration
from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.models.enums import CheckResult
from checkov.common.output.record import SCA_PACKAGE_SCAN_CHECK_NAME
from checkov.common.util.file_utils import convert_to_unix_path
from checkov.common.util.str_utils import removeprefix, align_path

if TYPE_CHECKING:
    from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration
    from checkov.common.output.report import Report
    from checkov.common.output.record import Record
    from checkov.common.typing import _BaseRunner


class SuppressionsIntegration(BaseIntegrationFeature):
    def __init__(self, bc_integration: BcPlatformIntegration) -> None:
        super().__init__(bc_integration=bc_integration, order=2)  # must be after the custom policies integration
        self.suppressions_v2: dict[str, list[dict[str, Any]]] = {}
        self.suppressions: dict[str, list[dict[str, Any]]] = {}

        # bcorgname_provider_timestamp (ex: companyxyz_aws_1234567891011)
        # the provider may be lower or upper depending on where the policy was created
        self.custom_policy_id_regex = re.compile(r'^[a-zA-Z0-9]+_[a-zA-Z]+_\d{13}$')
        self.repo_name_regex: Pattern[str] | None = None

    @property
    def suppressions_url(self) -> str:
        return f"{self.bc_integration.api_url}/api/v1/suppressions"

    def is_valid(self) -> bool:
        return (
            self.bc_integration.is_integration_configured()
            and not self.bc_integration.skip_download
            and not self.integration_feature_failures
        )

    def pre_scan(self) -> None:
        try:
            if not self.bc_integration.customer_run_config_response:
                logging.debug('In the pre-scan for suppressions, but nothing was fetched from the platform')
                self.integration_feature_failures = True
                return

            suppressions = self.bc_integration.customer_run_config_response.get('suppressions')
            suppressions_v2 = self.bc_integration.customer_run_config_response.get('suppressionsV2')  # currently just SAST

            for suppression in suppressions:
                suppression['isV1'] = True
                if suppression['policyId'] in metadata_integration.bc_to_ckv_id_mapping:
                    suppression['checkovPolicyId'] = metadata_integration.get_ckv_id_from_bc_id(suppression['policyId'])
                else:
                    suppression['checkovPolicyId'] = suppression['policyId']  # custom policy

            for suppression in suppressions_v2:
                suppression['isV1'] = False
                checkov_ids = []
                for policy_id in suppression['policyIds']:
                    if policy_id in metadata_integration.bc_to_ckv_id_mapping:
                        checkov_ids.append(metadata_integration.bc_to_ckv_id_mapping[policy_id])
                    else:
                        checkov_ids.append(policy_id)  # custom policy - not supported yet
                suppression['checkovPolicyIds'] = checkov_ids

            self._init_repo_regex()
            suppressions = sorted(suppressions, key=lambda s: s['checkovPolicyId'])

            # group and map by policy ID
            self.suppressions = {policy_id: list(sup) for policy_id, sup in
                                 groupby(suppressions, key=lambda s: s['checkovPolicyId'])}

            # map suppressions v2 by checkov ID - because the policy IDs are arrays, we need to map each unique ID in each
            # suppression's policy ID array to its suppressions
            self.suppressions_v2 = SuppressionsIntegration.create_suppression_v2_policy_id_map(suppressions_v2)

            logging.debug('The found suppression v1 rules are:')
            logging.debug(self.suppressions)
            logging.debug('The found suppression v2 rules are:')
            logging.debug(self.suppressions_v2)

        except Exception:
            self.integration_feature_failures = True
            logging.debug("Scanning without applying suppressions configured in the platform.", exc_info=True)

    @staticmethod
    def create_suppression_v2_policy_id_map(suppressions_v2: list[dict[str, Any]]) -> dict[str, list[dict[str, Any]]]:
        checkov_id_map: dict[str, list[dict[str, Any]]] = {}
        for suppression in suppressions_v2:
            for checkov_id in suppression['checkovPolicyIds']:
                if checkov_id in checkov_id_map:
                    checkov_id_map[checkov_id].append(suppression)
                else:
                    checkov_id_map[checkov_id] = [suppression]
        return checkov_id_map

    def post_runner(self, scan_report: Report) -> None:
        self._apply_suppressions_to_report(scan_report)

    def _apply_suppressions_to_report(self, scan_report: Report) -> None:

        # holds the checks that are still not suppressed
        still_failed_checks = []
        still_passed_checks = []
        for check in scan_report.failed_checks + scan_report.passed_checks:
            # in order to be able to suppress by policy we assign the relevant check id for package / image scan
            # and avoiding licenses vulns
            if scan_report.check_type == CheckType.SCA_PACKAGE and check.check_name == SCA_PACKAGE_SCAN_CHECK_NAME:
                check.check_id = 'BC_VUL_2'
            if scan_report.check_type == CheckType.SCA_IMAGE and check.check_name == SCA_PACKAGE_SCAN_CHECK_NAME:
                check.check_id = 'BC_VUL_1'

            relevant_suppressions = self.suppressions.get(check.check_id)
            relevant_suppressions_v2 = self.suppressions_v2.get(check.check_id)

            has_suppression = relevant_suppressions or relevant_suppressions_v2

            applied_suppression = self._check_suppressions(check, relevant_suppressions, relevant_suppressions_v2) if has_suppression else None
            if applied_suppression:
                suppress_comment = applied_suppression['comment'] if applied_suppression['isV1'] else applied_suppression['justificationComment']
                if self._should_omit_check(applied_suppression):
                    logging.debug(f'Removing check {check.check_id} from the report, comment: {suppress_comment}')
                else:
                    logging.debug(f'Applying suppression to the check {check.check_id} with the comment: {suppress_comment}')
                    check.check_result = {
                        'result': CheckResult.SKIPPED,
                        'suppress_comment': suppress_comment
                    }
                    scan_report.skipped_checks.append(check)
            elif check.check_result['result'] == CheckResult.FAILED:
                still_failed_checks.append(check)
            else:
                still_passed_checks.append(check)

        scan_report.failed_checks = still_failed_checks
        scan_report.passed_checks = still_passed_checks

    @staticmethod
    def _should_omit_check(applied_suppression: dict[str, Any]) -> bool:
        if applied_suppression['isV1']:
            return False
        if applied_suppression['ruleType'] == 'policy':
            return True
        return False

    def _check_suppressions(self, record: Record, suppressions: Optional[list[dict[str, Any]]], suppressions_v2: Optional[list[dict[str, Any]]]) -> dict[str, Any] | None:
        """
        Checks the specified suppressions against the specified record, returning the applied suppression, if any, else None
        :return:
        """
        if suppressions:
            for suppression in suppressions:
                if self._check_suppression(record, suppression):
                    return suppression
        if suppressions_v2:
            for suppression in suppressions_v2:
                if self._check_suppression_v2(record, suppression):
                    return suppression
        return None

    def _check_suppression(self, record: Record, suppression: dict[str, Any]) -> bool:
        """
        Returns True if and only if the specified suppression applies to the specified record.
        :param record:
        :param suppression:
        :return:
        """
        if record.check_id != suppression['checkovPolicyId']:
            return False

        type = suppression['suppressionType']

        if type == 'Policy':
            # We already validated the policy ID above
            return True
        elif type == 'Accounts':
            # This should be true, because we validated when we downloaded the policies.
            # But checking here adds some resiliency against bugs if that changes.
            return any(self.bc_integration.repo_matches(account) for account in suppression['accountIds'])
        elif type == 'Resources':
            for resource in suppression['resources']:
                if self.bc_integration.repo_matches(resource['accountId']) \
                        and (resource['resourceId'] == f'{record.repo_file_path}:{record.resource}'
                             or resource['resourceId'] == f'{convert_to_unix_path(record.file_path)}:{record.resource}'):
                    return True
            return False
        elif type == 'Tags':
            entity_tags = record.entity_tags
            if not entity_tags:
                return False
            suppression_tags = suppression['tags']  # a list of objects of the form {key: str, value: str}

            for tag in suppression_tags:
                key = tag['key']
                value = tag['value']
                if entity_tags.get(key) == value:
                    return True

        elif type == 'CvesAccounts':
            if 'accountIds' not in suppression:
                return False
            if self.bc_integration.source_id in suppression['accountIds']:
                if record.vulnerability_details and record.vulnerability_details['id'] in suppression['cves']:
                    return True
            return False

        elif type == 'Cves':
            if 'accountIds' not in suppression:
                return False
            if self.bc_integration.repo_id and self.bc_integration.source_id and self.bc_integration.source_id in suppression['accountIds']\
                    and suppression['cves']:
                repo_name = align_path(self.bc_integration.repo_id).split('/')[-1]
                suppression_path = self._get_cve_suppression_path(suppression)
                repo_file_path = align_path(record.repo_file_path)
                file_abs_path = align_path(record.file_abs_path)
                if file_abs_path == suppression_path[1:] or \
                        file_abs_path == suppression_path or \
                        file_abs_path.endswith("".join([repo_name, suppression_path])) or \
                        removeprefix(repo_file_path, '/') == removeprefix(suppression_path, '/'):
                    return any(record.vulnerability_details and record.vulnerability_details['id'] == cve['cve']
                               for cve in suppression['cves'])
            return False

        elif type == 'LicenseType':
            return any(record.vulnerability_details and record.vulnerability_details['license'] == license_type
                       for license_type in suppression.get('licenseTypes', []))

        return False

    @staticmethod
    def normalize_file_path(file_path: str) -> str:
        """
        Returns the file path with a leading slash, if not already present
        """
        return file_path if file_path.startswith('/') else f'/{file_path}'

    def _check_suppression_v2_file(self, record_file_path: str, suppression_file_path: str, suppression_repo_name: str) -> bool:
        return self.bc_integration.repo_matches(suppression_repo_name)\
            and (suppression_file_path == record_file_path or suppression_file_path == convert_to_unix_path(record_file_path))

    def _check_suppression_v2(self, record: Record, suppression: dict[str, Any]) -> bool:
        if record.check_id not in suppression['checkovPolicyIds']:
            return False

        type = suppression['ruleType']

        if type == 'policy':
            # We just checked the policy ID above
            return True
        elif type == 'finding':
            pass  # TODO how to map them?
        elif type == 'file':
            record_file_path = SuppressionsIntegration.normalize_file_path(record.repo_file_path)
            for file_suppression in suppression['files']:
                suppression_file_path = SuppressionsIntegration.normalize_file_path(file_suppression['filePath'])
                if self._check_suppression_v2_file(record_file_path, suppression_file_path, file_suppression.get('repositoryName', '')):
                    return True
        elif type == 'repository':
            return any(self.bc_integration.repo_matches(repo.get('repositoryName', '')) for repo in suppression['repositories'])
        return False

    def _get_cve_suppression_path(self, suppression: dict[str, Any]) -> str:
        suppression_path: str = align_path(suppression['cves'][0]['id'])
        # for handling cases of IR/docker (e.g: '/Dockerfile:/DockerFile.FROM)
        suppression_path_parts = suppression_path.split(':')
        if len(suppression_path_parts) == 2 and suppression_path_parts[1].startswith(suppression_path_parts[0]):
            return suppression_path_parts[0]
        return suppression_path

    def _suppression_valid_for_run(self, suppression: dict[str, Any]) -> bool:
        """
        Returns whether this suppression is valid. A suppression is NOT valid if:
        - the policy does not have a checkov ID and does not have an ID matching a custom policy format
        - the suppression type is 'Accounts' and this repo is not included in the account list
        :param suppression:
        :return:
        """
        policyId = suppression['policyId']
        if policyId not in metadata_integration.bc_to_ckv_id_mapping and not self.custom_policy_id_regex.match(
                policyId):
            return False

        if suppression['suppressionType'] == 'Accounts':
            if not any(self.bc_integration.repo_matches(account) for account in suppression['accountIds']):
                return False

        return True

    def _repo_matches(self, repo_name: str) -> bool:
        if not self.repo_name_regex:
            # shouldn't happen
            return False

        # matches xyz_org/repo or org/repo (where xyz is the BC org name and the CLI repo prefix from the platform)
        return self.repo_name_regex.match(repo_name) is not None

    def _init_repo_regex(self) -> None:
        self.repo_name_regex = re.compile(f'^([a-zA-Z0-9]+_)?{self.bc_integration.repo_id}$')

    def pre_runner(self, runner: _BaseRunner) -> None:
        # not used
        pass

    def get_policy_level_suppressions(self) -> dict[str, list[str]]:
        policy_level_suppressions = {}
        for check_suppressions in itertools.chain(self.suppressions.values(), self.suppressions_v2.values()):
            for suppression in check_suppressions:
                if (suppression['isV1'] and suppression.get("suppressionType") == "Policy") or (not suppression['isV1'] and suppression.get("ruleType") == "policy"):
                    policy_level_suppressions[suppression['id']] = [suppression['policyId']] if suppression['isV1'] else suppression['policyIds']
        return policy_level_suppressions

    def post_scan(self, merged_reports: list[Report]) -> None:
        # not used
        pass


integration = SuppressionsIntegration(bc_integration)



from __future__ import annotations

from typing import List, Optional, Dict, Any, Tuple, TYPE_CHECKING
from checkov.common.checks_infra.solvers import ConnectionExistsSolver
from checkov.common.graph.checks_infra import debug
from checkov.common.graph.checks_infra.enums import Operators

if TYPE_CHECKING:
    from checkov.common.typing import LibraryGraph


class ConnectionOneExistsSolver(ConnectionExistsSolver):
    operator = Operators.ONE_EXISTS  # noqa: CCE003  # a static attribute

    def __init__(
        self,
        resource_types: List[str],
        connected_resources_types: List[str],
        vertices_under_resource_types: Optional[List[Dict[str, Any]]] = None,
        vertices_under_connected_resources_types: Optional[List[Dict[str, Any]]] = None,
    ) -> None:
        super().__init__(
            resource_types,
            connected_resources_types,
            vertices_under_resource_types,
            vertices_under_connected_resources_types,
        )

    def get_operation(self, graph_connector: LibraryGraph) -> \
            Tuple[List[Dict[str, Any]], List[Dict[str, Any]], List[Dict[str, Any]]]:
        passed, failed, unknown = super()._get_operation(graph_connector)
        failed = [f for f in failed if f not in passed]
        unknown = [u for u in unknown if u not in passed]

        debug.connection_block(
            resource_types=self.resource_types,
            connected_resource_types=self.connected_resources_types,
            operator=self.operator,
            passed_resources=passed,
            failed_resources=failed,
        )

        return passed, failed, unknown



from __future__ import annotations

import re
from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.module.base_module_check import BaseModuleCheck
from .RevisionHash import check as RevisionHashCheck

VERSION_PATTERN = re.compile(r"[?&](ref=).*(\d\.\d).*")


class RevisionVersionTag(BaseModuleCheck):
    def __init__(self) -> None:
        name = "Ensure Terraform module sources use a tag with a version number"
        id = "CKV_TF_2"
        categories = [CheckCategories.SUPPLY_CHAIN]
        super().__init__(name=name, id=id, categories=categories)

    def scan_module_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        # this check is a more lenient version of the hash check, so if that one passes (or is unknown due to a local module),
        # then we are done
        hash_result = RevisionHashCheck.scan_module_conf(conf)
        if hash_result != CheckResult.FAILED:
            return hash_result

        source = conf.get("source")
        if source and isinstance(source, list):
            source_url = source[0]
            if BaseModuleCheck.is_git_source(source_url):
                if ("?ref" in source_url or "&ref" in source_url) and re.search(VERSION_PATTERN, source_url):
                    return CheckResult.PASSED
            else:
                version = conf.get("version")
                if source and isinstance(version, list):
                    return CheckResult.PASSED

        return CheckResult.FAILED


check = RevisionVersionTag()



from typing import Any, List

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class CloudFunctionPermissiveIngress(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure GCP Cloud Function is not configured with overly permissive Ingress setting"
        id = "CKV_GCP_124"
        supported_resources = ("google_cloudfunctions_function", "google_cloudfunctions2_function")
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        if self.entity_type == "google_cloudfunctions_function":
            return "ingress_settings"
        else:
            return "service_config/[0]/ingress_settings/[0]"

    def get_expected_values(self) -> List[Any]:
        return ["ALLOW_INTERNAL_AND_GCLB", "ALLOW_INTERNAL_ONLY"]


check = CloudFunctionPermissiveIngress()



from typing import Dict, Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.util.consts import START_LINE, END_LINE


class AzureMLWorkspacePrivateEndpoint(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Azure Machine learning workspace is not configured with private endpoint"
        id = "CKV_AZURE_239"
        supported_resources = ["Microsoft.MachineLearningServices/workspaces"]
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        properties = conf.get("properties")
        if isinstance(properties, dict):
            managed_network = properties.get("managedNetwork")
            if isinstance(managed_network, dict):
                ob_rules = managed_network.get("outboundRules")
                if isinstance(ob_rules, dict):
                    # check no outbound rule has private endpoint type
                    for key, rule in ob_rules.items():
                        if key in [START_LINE, END_LINE]:
                            # Skip inner fields we add
                            continue
                        if rule.get("type") == "PrivateEndpoint":
                            return CheckResult.FAILED
        return CheckResult.PASSED


check = AzureMLWorkspacePrivateEndpoint()



from __future__ import annotations

from typing import Optional

FOREACH_KEY_SEPERATOR = '["'
FOREACH_KEY_ENDER = '"]'
COUNT_KEY_SEPERATOR = "["
COUNT_KEY_ENDER = "]"


def get_terraform_foreach_or_count_key(resource_id: str) -> Optional[str]:
    sanitized_id = get_sanitized_terraform_resource_id(resource_id)
    if sanitized_id == resource_id:
        return None
    key = resource_id.split(sanitized_id)[-1]
    while key.startswith(FOREACH_KEY_SEPERATOR) and key.endswith(FOREACH_KEY_ENDER):
        key = key[2:-2]
    while key.startswith(COUNT_KEY_SEPERATOR) and key.endswith(COUNT_KEY_ENDER):
        key = key[1:-1]
    return key


def get_sanitized_terraform_resource_id(resource_id: str) -> str:
    if FOREACH_KEY_SEPERATOR in resource_id:
        original_id_parts = resource_id.split(FOREACH_KEY_SEPERATOR, maxsplit=1)
        original_resource_name = original_id_parts[-2]  # As the last item will be the key itself,
        return original_resource_name  # This will be the resource id before the foreach key was added
    elif COUNT_KEY_SEPERATOR in resource_id:
        original_id_parts = resource_id.split(COUNT_KEY_SEPERATOR)
        original_resource_name = original_id_parts[-2]
        return original_resource_name
    return resource_id



from typing import Dict, List, Any, Optional, Union
from pydantic import BaseModel, model_serializer

from checkov.common.sast.consts import SastLanguages


class Profiler(BaseModel):
    duration: Union[str, int]  # noqa: CCE003
    memory: int  # noqa: CCE003


class Point(BaseModel):
    row: int  # noqa: CCE003
    column: int  # noqa: CCE003

    @model_serializer
    def serialize_model(self) -> Dict[str, Any]:
        return self.__dict__


class MatchLocation(BaseModel):
    path: str  # noqa: CCE003
    start: Point  # noqa: CCE003
    end: Point  # noqa: CCE003
    code_block: str  # noqa: CCE003
    extended_code_block: Optional[Dict[str, Any]] = None   # noqa: CCE003

    @model_serializer
    def serialize_model(self) -> Dict[str, Any]:
        return self.__dict__


class DataFlow(BaseModel):
    data_flow: List[MatchLocation]  # noqa: CCE003

    @model_serializer
    def serialize_model(self) -> Dict[str, List[MatchLocation]]:
        return {'data_flow': self.data_flow}


class MatchMetadata(BaseModel):
    taint_mode: Optional[DataFlow] = None  # noqa: CCE003
    code_locations: Optional[List[MatchLocation]] = None  # noqa: CCE003

    @model_serializer
    def serialize_model(self) -> Dict[str, Any]:
        metadata = {}
        if hasattr(self, 'taint_mode') and self.taint_mode:
            metadata['taint_mode'] = self.taint_mode
        if hasattr(self, 'code_locations') and self.code_locations:
            metadata['code_locations'] = self.code_locations  # type: ignore

        return metadata


class Match(BaseModel):
    exact_hash: str  # noqa: CCE003
    structure_hash: str  # noqa: CCE003
    location: MatchLocation  # noqa: CCE003
    metadata: MatchMetadata  # noqa: CCE003
    minimized_ast: str  # noqa: CCE003


class RuleMatchMetadata(BaseModel):
    framework: Optional[str] = ''  # noqa: CCE003


class RuleMatch(BaseModel):
    check_id: str  # noqa: CCE003
    check_name: str  # noqa: CCE003
    check_cwe: Optional[Union[List[str], str]]  # noqa: CCE003
    check_owasp: Optional[Union[List[str], str]]  # noqa: CCE003
    severity: str  # noqa: CCE003
    matches: List[Match]  # noqa: CCE003
    metadata: Optional[RuleMatchMetadata] = None  # noqa: CCE003


class Function(BaseModel):
    name: str  # noqa: CCE003
    alias: str  # noqa: CCE003
    line_number: int  # noqa: CCE003
    code_block: str  # noqa: CCE003
    cve_id: Optional[str]  # noqa: CCE003

    def to_dict(self) -> Dict[str, Any]:
        return self.__dict__


class Package(BaseModel):
    alias: str  # noqa: CCE003
    functions: List[Function]   # noqa: CCE003


class File(BaseModel):
    packages: Dict[str, Package]  # noqa: CCE003


class Repositories(BaseModel):
    files: Dict[str, File]  # noqa: CCE003


class SkippedCheck(BaseModel):
    check_id: str   # noqa: CCE003
    suppress_comment: str   # noqa: CCE003


class PrismaReport(BaseModel):
    rule_match: Dict[SastLanguages, Dict[str, RuleMatch]]  # noqa: CCE003
    errors: Dict[str, List[str]]  # noqa: CCE003
    profiler: Dict[str, Profiler]  # noqa: CCE003
    run_metadata: Dict[str, Optional[Union[str, int, List[str]]]]  # noqa: CCE003
    imports: Dict[SastLanguages, Dict[str, Dict[str, Union[List[str], Dict[str, str]]]]]  # noqa: CCE003
    reachability_report: Dict[SastLanguages, Dict[str, Repositories]]   # noqa: CCE003
    skipped_checks_by_file: Dict[str, List[SkippedCheck]]   # noqa: CCE003


def create_empty_report(languages: List[SastLanguages]) -> PrismaReport:
    matches: Dict[SastLanguages, Dict[str, RuleMatch]] = {}
    for lang in languages:
        matches[lang] = {}
    return PrismaReport(rule_match=matches, errors={}, profiler={}, run_metadata={}, imports={}, reachability_report={},
                        skipped_checks_by_file={})


def serialize_reachability_report(report: Dict[str, Repositories]) -> Dict[str, Any]:
    result: Dict[str, Any] = {}
    for repo_path, files in report.items():
        result[repo_path] = {"files": {}}
        for file_name, packages in files.files.items():
            result[repo_path]["files"][file_name] = {"packages": {}}
            for package_name, package in packages.packages.items():
                result[repo_path]["files"][file_name]["packages"][package_name] = {"alias": package.alias, "functions": []}
                for function in package.functions:
                    result[repo_path]["files"][file_name]["packages"][package_name]["functions"].append(function.to_dict())
    return result



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class CloudfrontDistributionEncryption(BaseResourceCheck):
    def __init__(self):
        name = "Ensure CloudFront Distribution ViewerProtocolPolicy is set to HTTPS"
        id = "CKV_AWS_34"
        supported_resources = ['AWS::CloudFront::Distribution']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for ViewerProtocolPolicy configuration at cloudfront distributions:
            https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distribution-cachebehavior.html
            https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distribution-defaultcachebehavior.html
        :param conf: cloudfront configuration
        :return: <CheckResult>
        """

        if 'Properties' in conf.keys():
            if 'DistributionConfig' in conf['Properties'].keys():
                if 'DefaultCacheBehavior' in conf['Properties']['DistributionConfig'].keys():
                    if 'ViewerProtocolPolicy' in conf['Properties']['DistributionConfig']['DefaultCacheBehavior'].keys():
                        if conf['Properties']['DistributionConfig']['DefaultCacheBehavior']['ViewerProtocolPolicy'] == 'allow-all':
                            return CheckResult.FAILED
                if 'CacheBehaviors' in conf['Properties']['DistributionConfig'].keys():
                    if not isinstance(conf['Properties']['DistributionConfig']['CacheBehaviors'], list):
                        return CheckResult.UNKNOWN
                    for behavior in range(len(conf['Properties']['DistributionConfig']['CacheBehaviors'])):
                        if 'ViewerProtocolPolicy' in conf['Properties']['DistributionConfig']['CacheBehaviors'][behavior].keys():
                            if conf['Properties']['DistributionConfig']['CacheBehaviors'][behavior]['ViewerProtocolPolicy'] == 'allow-all':
                                return CheckResult.FAILED
        return CheckResult.PASSED


check = CloudfrontDistributionEncryption()



from checkov.arm.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class FunctionAppHttpVersionLatest(BaseResourceValueCheck):

    def __init__(self) -> None:
        name = "Ensure that 'HTTP Version' is the latest, if used to run the Function app"
        id = "CKV_AZURE_67"
        supported_resources = ("Microsoft.Web/sites/slots", "Microsoft.Web/sites",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "properties/siteConfig/http20Enabled"


check = FunctionAppHttpVersionLatest()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck

LOG_TYPES = ("profiler", "audit")


class DocDBLogging(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure DocumentDB Logging is enabled"
        id = "CKV_AWS_85"
        supported_resources = ("aws_docdb_cluster",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        logs_exports = conf.get("enabled_cloudwatch_logs_exports")
        if logs_exports and isinstance(logs_exports, list):
            if any(elem in logs_exports[0] for elem in LOG_TYPES):
                return CheckResult.PASSED
        return CheckResult.FAILED

    def get_evaluated_keys(self) -> list[str]:
        return ["enabled_cloudwatch_logs_exports"]


check = DocDBLogging()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class PostgreSQLServerHasPublicAccessDisabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that PostgreSQL server disables public network access"
        id = "CKV_AZURE_68"
        supported_resources = ('Microsoft.DBforPostgreSQL/servers',)
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.FAILED)

    def get_inspected_key(self) -> str:
        return 'properties/publicNetworkAccess'

    def get_expected_value(self) -> str:
        return "Disabled"


check = PostgreSQLServerHasPublicAccessDisabled()



from enum import Enum
from typing import Optional, Dict

from checkov.common.sast.consts import SastLanguages


class ScaPackageFile(Enum):
    PACKAGE_JSON = 'package.json'
    PACKAGE_JSON_LOCK = 'package-lock.json'
    POM_XML = 'pom.xml'
    BUILD_GRADLE = 'build.gradle'
    PIPFILE = 'Pipfile'
    PIPFILE_LOCK = 'Pipfile.lock'


sca_package_to_sast_lang_map: Dict[ScaPackageFile, SastLanguages] = {
    ScaPackageFile.PACKAGE_JSON: SastLanguages.JAVASCRIPT,
    ScaPackageFile.PACKAGE_JSON_LOCK: SastLanguages.JAVASCRIPT,
    ScaPackageFile.POM_XML: SastLanguages.JAVA,
    ScaPackageFile.BUILD_GRADLE: SastLanguages.JAVA,
    ScaPackageFile.PIPFILE: SastLanguages.PYTHON,
    ScaPackageFile.PIPFILE_LOCK: SastLanguages.PYTHON
}


def get_package_by_str(package_name: str) -> Optional[ScaPackageFile]:
    for enum_member in ScaPackageFile:
        if enum_member.value == package_name:
            return enum_member
    return None



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck

LOG_TYPES = ("profiler", "audit")


class DocDBLogging(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure DocumentDB Logging is enabled"
        id = "CKV_AWS_85"
        supported_resources = ("AWS::DocDB::DBCluster",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        properties = conf.get("Properties")
        if properties:
            logs_exports = properties.get("EnableCloudwatchLogsExports")
            if logs_exports:
                if any(elem in logs_exports for elem in LOG_TYPES):
                    return CheckResult.PASSED
        return CheckResult.FAILED

    def get_evaluated_keys(self) -> list[str]:
        return ["Properties/EnableCloudwatchLogsExports"]


check = DocDBLogging()



from __future__ import annotations
from typing import Any
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class CosmosDBLocalAuthDisabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        description = "Ensure that Local Authentication is disabled on CosmosDB"
        id = "CKV_AZURE_140"
        supported_resources = ('Microsoft.DocumentDB/databaseAccounts',)
        categories = (CheckCategories.IAM,)
        super().__init__(name=description, id=id, categories=categories, supported_resources=supported_resources,)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        if conf.get("kind") == "GlobalDocumentDB":
            return super().scan_resource_conf(conf)
        return CheckResult.UNKNOWN

    def get_inspected_key(self) -> str:
        return "properties/disableLocalAuth"

    def get_expected_value(self) -> bool:
        return True


check = CosmosDBLocalAuthDisabled()



from typing import Any, Dict

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck


class HostPort(BaseK8sContainerCheck):
    def __init__(self) -> None:
        """
        https://kubernetes.io/docs/concepts/configuration/overview/

        Don’t specify a hostPort for a Pod unless it is absolutely necessary.
        When you bind a Pod to a hostPort, it limits the number of places the
        Pod can be scheduled, because each <hostIP, hostPort, protocol> combination
        must be unique.
        """
        name = "Do not specify hostPort unless absolutely necessary"
        id = "CKV_K8S_26"
        # Location: container .ports[].hostPort
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        if conf.get("ports"):
            for idx, port in enumerate(conf["ports"]):
                if "hostPort" in port:
                    self.evaluated_container_keys = [f"ports/[{idx}]/hostPort"]
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = HostPort()



from typing import Any, List

from checkov.arm.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class AppServiceDisallowCORS(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure that CORS disallows every resource to access app services"
        id = "CKV_AZURE_57"
        supported_resources = ("Microsoft.Web/sites",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            missing_block_result=CheckResult.PASSED
        )

    def get_inspected_key(self) -> str:
        return 'properties/siteConfig/cors/allowedOrigins'

    def get_forbidden_values(self) -> List[Any]:
        return ['*']


check = AppServiceDisallowCORS()



from __future__ import annotations

import json
import logging
import re
from collections import defaultdict
import tempfile
from typing import TYPE_CHECKING, Any, List

from checkov.common.bridgecrew.integration_features.base_integration_feature import BaseIntegrationFeature
from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.bridgecrew.severities import Severities
from checkov.common.checks_infra.checks_parser import GraphCheckParser
from checkov.common.checks_infra.registry import Registry, get_graph_checks_registry
from checkov.common.util.data_structures_utils import pickle_deepcopy

if TYPE_CHECKING:
    from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration
    from checkov.common.output.record import Record
    from checkov.common.output.report import Report
    from checkov.common.typing import _BaseRunner

# service-provider::service-name::data-type-name
CFN_RESOURCE_TYPE_IDENTIFIER = re.compile(r"^[a-zA-Z0-9]+::[a-zA-Z0-9]+::[a-zA-Z0-9]+$")
SAST_CATEGORY = 'Sast'
LICENSES_CATEGORY = 'Licenses'


class CustomPoliciesIntegration(BaseIntegrationFeature):
    def __init__(self, bc_integration: BcPlatformIntegration) -> None:
        super().__init__(bc_integration=bc_integration, order=1)  # must be after policy metadata and before suppression integration
        self.platform_policy_parser = GraphCheckParser()
        self.bc_cloned_checks: dict[str, list[dict[str, Any]]] = defaultdict(list)
        self.policy_level_suppression: List[str] = []

    @property
    def policies_url(self) -> str:
        return f"{self.bc_integration.api_url}/api/v1/policies/table/data"

    def is_valid(self) -> bool:
        return (
            self.bc_integration.is_integration_configured()
            and not self.bc_integration.skip_download
            and not self.integration_feature_failures
        )

    def pre_scan(self) -> None:
        try:
            if not self.bc_integration.customer_run_config_response:
                logging.debug('In the pre-scan for custom policies, but nothing was fetched from the platform')
                self.integration_feature_failures = True
                return

            policies = self.bc_integration.customer_run_config_response.get('customPolicies')
            sast_policies_dir = tempfile.mkdtemp()
            self.bc_integration.sast_custom_policies = sast_policies_dir
            for policy in policies:
                try:
                    logging.debug(f"Loading policy id: {policy.get('id')}")
                    if policy.get('category') == SAST_CATEGORY:
                        with open(f"{sast_policies_dir}/{policy.get('id')}.yaml", 'a') as f:
                            f.write(policy.get('code'))
                        continue

                    converted_check = self._convert_raw_check(policy)
                    source_incident_id = policy.get('sourceIncidentId')
                    if source_incident_id:
                        policy['severity'] = Severities[policy['severity']]
                        self.bc_cloned_checks[source_incident_id].append(policy)
                        continue
                    resource_types = Registry._get_resource_types(converted_check['metadata'])

                    if policy.get('category') == LICENSES_CATEGORY:
                        continue

                    check = self.platform_policy_parser.parse_raw_check(converted_check, resources_types=resource_types)
                    check.severity = Severities[policy['severity']]
                    check.bc_id = check.id
                    if check.frameworks:
                        for f in check.frameworks:
                            if f.lower() == "cloudformation":
                                get_graph_checks_registry("cloudformation").checks.append(check)
                            elif f.lower() == "terraform":
                                get_graph_checks_registry("terraform").checks.append(check)
                            elif f.lower() == "kubernetes":
                                get_graph_checks_registry("kubernetes").checks.append(check)
                    elif re.match(CFN_RESOURCE_TYPE_IDENTIFIER, check.resource_types[0]):
                        get_graph_checks_registry("cloudformation").checks.append(check)
                    else:
                        get_graph_checks_registry("terraform").checks.append(check)
                except Exception:
                    logging.debug(f"Failed to load policy id: {policy.get('id')}", exc_info=True)
            logging.debug(f'Found {len(policies)} custom policies from the platform.')
        except Exception:
            self.integration_feature_failures = True
            logging.debug("Scanning without applying custom policies from the platform.", exc_info=True)

    @staticmethod
    def _convert_raw_check(policy: dict[str, Any]) -> dict[str, Any]:
        metadata = {
            'id': policy['id'],
            'name': policy['title'],
            'category': policy['category'],
            'frameworks': policy.get('frameworks', [])
        }
        check = {
            'metadata': metadata,
            'definition': json.loads(policy['code'])
        }
        return check

    def post_runner(self, scan_report: Report) -> None:
        if self.bc_cloned_checks:
            scan_report.failed_checks = self.extend_records_with_cloned_policies(scan_report.failed_checks)
            scan_report.passed_checks = self.extend_records_with_cloned_policies(scan_report.passed_checks)
            scan_report.skipped_checks = self.extend_records_with_cloned_policies(scan_report.skipped_checks)

    def extend_records_with_cloned_policies(self, records: list[Record]) -> list[Record]:
        bc_check_ids = [record.bc_check_id for record in records]
        for idx, bc_check_id in enumerate(bc_check_ids):
            cloned_policies = self.bc_cloned_checks.get(bc_check_id, [])  # type:ignore[arg-type]  # bc_check_id can be None
            logging.debug('Cloned policies to be deep copied:')
            logging.debug(cloned_policies)
            logging.debug('From origin policy:')
            logging.debug(records[idx].get_unique_string())
            for cloned_policy in cloned_policies:
                new_record = pickle_deepcopy(records[idx])
                new_record.check_id = cloned_policy['id']
                new_record.bc_check_id = cloned_policy['id']
                new_record.guideline = cloned_policy['guideline']
                new_record.severity = cloned_policy['severity']
                new_record.check_name = cloned_policy['title']
                records.append(new_record)
        policy_level_suppression_check_ids = self.convert_suppression_ids_to_bc_check_ids()
        records = [record for record in records if record.bc_check_id not in policy_level_suppression_check_ids]  # Filter out policy level suppressions after cloned policy is added
        return records

    def convert_suppression_ids_to_bc_check_ids(self) -> List[str]:
        return ["_".join(policy.split('_')[:-1]) for policy in self.policy_level_suppression]

    def pre_runner(self, runner: _BaseRunner) -> None:
        # not used
        pass

    def post_scan(self, merged_reports: list[Report]) -> None:
        # not used
        pass


integration = CustomPoliciesIntegration(bc_integration)



from typing import Any, Dict, List
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_check import BaseResourceCheck


class AzureDefenderOnStorage(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that Azure Defender is set to On for Storage"
        id = "CKV_AZURE_84"
        supported_resources = ("Microsoft.Security/pricings",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        properties: Dict[str, Any] = conf.get("properties", {})
        pricingTier = properties.get("pricingTier", "")
        return (
            CheckResult.PASSED
            if pricingTier == "Standard"
            else CheckResult.FAILED
        )

    def get_evaluated_keys(self) -> List[str]:
        return ["properties/pricingTier"]


check = AzureDefenderOnStorage()



from typing import Dict, Any
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_list


class AppGatewayWAFACLCVE202144228(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Application Gateway WAF prevents message lookup in Log4j2. See CVE-2021-44228 aka log4jshell"
        id = "CKV_AZURE_135"
        supported_resources = ("Microsoft.Network/ApplicationGatewayWebApplicationFirewallPolicies",)
        categories = (CheckCategories.APPLICATION_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        properties = conf.get("properties")
        if not properties:
            return CheckResult.FAILED
        self.evaluated_keys = properties.get("managedRules")
        managed_rules = properties.get("managedRules")
        if managed_rules:
            managed_rule_sets = managed_rules.get("managedRuleSets") or []
            for idx_rule_set, rule_set in enumerate(force_list(managed_rule_sets)):
                self.evaluated_keys = [
                    f"managedRules/[0]/managedRuleSets[{idx_rule_set}]/ruleSetType",
                    f"managedRules/[0]/managedRuleSets[{idx_rule_set}]/ruleSetVersion",
                ]
                if (rule_set.get("ruleSetType") == "OWASP" or not rule_set.get("ruleSetType")) and rule_set.get("ruleSetVersion") in ["3.1", "3.2"]:
                    rule_overrides = rule_set.get("ruleGroupOverrides") or []
                    for idx_override, rule_override in enumerate(force_list(rule_overrides)):
                        self.evaluated_keys.extend(
                            [
                                f"managedRules/[0]/managedRuleSets[{idx_rule_set}]/ruleGroupOverrides/[{idx_override}]/ruleGroupName",
                                f"managedRules/[0]/managedRuleSets[{idx_rule_set}]/ruleGroupOverrides/[{idx_override}]/rules",
                            ]
                        )
                        if isinstance(rule_override, dict) and rule_override.get("ruleGroupName") == "REQUEST-944-APPLICATION-ATTACK-JAVA":
                            disabled_rules = rule_override.get("rules") or []
                            for idx_rule_id, disabled_rule in enumerate(force_list(disabled_rules)):
                                self.evaluated_keys.extend(
                                    [
                                        f"managedRules/[0]/managedRuleSets[{idx_rule_set}]/ruleGroupOverrides/[{idx_override}]/rules/[{idx_rule_id}]/ruleId",
                                    ]
                                )
                                if disabled_rule.get("ruleId") == "944240":
                                    return CheckResult.FAILED

                    return CheckResult.PASSED

        return CheckResult.FAILED


check = AppGatewayWAFACLCVE202144228()



from typing import Optional, Any, Dict

from checkov.common.graph.checks_infra.enums import Operators
from .greater_than_or_equal_attribute_solver import GreaterThanOrEqualAttributeSolver


class LessThanAttributeSolver(GreaterThanOrEqualAttributeSolver):
    operator = Operators.LESS_THAN  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        if vertex.get(attribute) is None:  # type:ignore[arg-type]  # due to attribute can be None
            return False

        return not super()._get_operation(vertex, attribute)



from checkov.common.models.enums import CheckCategories
from typing import Any
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class CloudsearchDomainTLS(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that CloudSearch is using latest TLS"
        id = "CKV_AWS_218"
        supported_resources = ("aws_cloudsearch_domain",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def get_inspected_key(self) -> str:
        return "endpoint_options/[0]/tls_security_policy"

    def get_expected_value(self) -> Any:
        return "Policy-Min-TLS-1-2-2019-07"


check = CloudsearchDomainTLS()



import logging
import requests
import os

from checkov.common.goget.base_getter import BaseGetter
from checkov.common.resource_code_logger_filter import add_resource_code_filter_to_logger
from checkov.common.util.file_utils import extract_tar_archive
from checkov.common.util.file_utils import extract_zip_archive
from checkov.common.util.http_utils import DEFAULT_TIMEOUT


class RegistryGetter(BaseGetter):
    def __init__(self, url: str, extension: str, create_clone_and_result_dirs: bool = False) -> None:
        self.logger = logging.getLogger(__name__)
        add_resource_code_filter_to_logger(self.logger)
        self.extension = extension
        self.create_clone_and_res_dirs = create_clone_and_result_dirs
        super().__init__(url)

    def do_get(self) -> str:
        # get dest dir
        download_path = os.path.join(self.temp_dir, f'module_source.{self.extension}')
        # download zip
        dest_path = os.path.dirname(download_path)
        with requests.get(self.url, stream=True, timeout=DEFAULT_TIMEOUT) as r:
            r.raise_for_status()
            os.makedirs(dest_path, exist_ok=True)
            with open(download_path, 'wb+') as f:
                for chunk in r.iter_content(chunk_size=8192):
                    f.write(chunk)
        # extract
        if self.extension == 'zip':
            extract_zip_archive(source_path=download_path, dest_path=dest_path)
        else:
            extract_tar_archive(source_path=download_path, dest_path=dest_path)
        os.remove(download_path)

        return dest_path



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class ElasticacheReplicationGroupEncryptionAtRest(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure all data stored in the ElastiCache Replication Group is securely encrypted at rest"
        id = "CKV_AWS_29"
        supported_resources = ['aws_elasticache_replication_group']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "at_rest_encryption_enabled"


check = ElasticacheReplicationGroupEncryptionAtRest()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class DataLakeStoreEncryption(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that Data Lake Store accounts enables encryption"
        id = "CKV_AZURE_105"
        supported_resources = ('azurerm_data_lake_store',)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources, missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self):
        return 'encryption_state'

    def get_expected_value(self):
        return "Enabled"


check = DataLakeStoreEncryption()



from __future__ import annotations

from typing import Any

from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.data_structures_utils import find_in_dict


class IMDSv1Disabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure Instance Metadata Service Version 1 is not enabled"
        id = "CKV_AWS_79"
        supported_resources = ("AWS::EC2::LaunchTemplate",)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        # IMDS can be disabled or IMDSv2 can be enabled
        http_endpoint = find_in_dict(
            input_dict=conf,
            key_path="Properties/LaunchTemplateData/MetadataOptions/HttpEndpoint",
        )
        if http_endpoint == "disabled":
            return CheckResult.PASSED

        return super().scan_resource_conf(conf)

    def get_inspected_key(self) -> str:
        return "Properties/LaunchTemplateData/MetadataOptions/HttpTokens"

    def get_expected_value(self) -> Any:
        return "required"


check = IMDSv1Disabled()



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class S3AbortIncompleteUploads(BaseResourceCheck):
    def __init__(self) -> None:
        """
        If you don't set this value in a lifecycle configuration you'll end up paying for s3
        resources you never could use
        """
        name = "Ensure S3 lifecycle configuration sets period for aborting failed uploads"
        id = "CKV_AWS_300"
        supported_resources = ("aws_s3_bucket_lifecycle_configuration",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        self.evaluated_keys = ["rule"]
        rules = conf.get("rule")
        if rules and isinstance(rules, list):
            for idx_rule, rule in enumerate(rules):
                if rule.get("abort_incomplete_multipart_upload") and rule.get("status") == ["Enabled"]:
                    self.evaluated_keys = [f"rule/[{idx_rule}]/abort_incomplete_multipart_upload"]
                    filter_list = rule.get("filter")
                    if filter_list and isinstance(filter_list, list):
                        # if filter is empty then rule applies to all paths so we pass
                        found_non_empty_parameter = False
                        for filter_item in filter_list:
                            if isinstance(filter_item, dict):       # check each filter parameter
                                connected = filter_item.get('and')
                                if connected and connected[0]:
                                    filter_item = connected[0]
                                prefix = filter_item.get('prefix')
                                if prefix and prefix[0]:
                                    found_non_empty_parameter = True
                                    continue
                                object_size_greater_than = filter_item.get('object_size_greater_than')
                                if object_size_greater_than and object_size_greater_than[0]:
                                    found_non_empty_parameter = True
                                    continue
                                object_size_less_than = filter_item.get('object_size_less_than')
                                if object_size_less_than and object_size_less_than[0]:
                                    found_non_empty_parameter = True
                                    continue
                                tag = filter_item.get('tag')
                                if tag and tag[0]:
                                    found_non_empty_parameter = True
                                    continue

                        if found_non_empty_parameter:       # continue searching for rules
                            continue

                    return CheckResult.PASSED

        return CheckResult.FAILED


check = S3AbortIncompleteUploads()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck
from typing import List


class CodebuildHasLogs(BaseResourceCheck):
    def __init__(self) -> None:
        """
        NIST.800-53.r5 AC-2(12), NIST.800-53.r5 AC-2(4), NIST.800-53.r5 AC-4(26), NIST.800-53.r5 AC-6(9),
        NIST.800-53.r5 AU-10, NIST.800-53.r5 AU-12, NIST.800-53.r5 AU-2, NIST.800-53.r5 AU-3, NIST.800-53.r5 AU-6(3),
        NIST.800-53.r5 AU-6(4), NIST.800-53.r5 AU-9(7), NIST.800-53.r5 CA-7, NIST.800-53.r5 SC-7(9),
        NIST.800-53.r5 SI-3(8), NIST.800-53.r5 SI-4, NIST.800-53.r5 SI-4(20), NIST.800-53.r5 SI-7(8)
        CodeBuild project environments should have a logging configuration
        """
        name = "Ensure CodeBuild project environments have a logging configuration"
        id = "CKV_AWS_314"
        supported_resources = ('aws_codebuild_project',)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        logs_config = conf.get('logs_config')
        if logs_config and isinstance(logs_config, list):
            logs = logs_config[0]
            if isinstance(logs, dict):
                if logs.get("cloudwatch_logs") or logs.get("s3_logs"):
                    return CheckResult.PASSED
        return CheckResult.FAILED

    def get_evaluated_keys(self) -> List[str]:
        return ['enabled_cloudwatch_logs_exports']


check = CodebuildHasLogs()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from typing import Any, List


class StorageAccountsUseReplication(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Storage Accounts use replication"
        id = "CKV_AZURE_206"
        supported_resources = ("azurerm_storage_account",)
        categories = (CheckCategories.BACKUP_AND_RECOVERY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "account_replication_type"

    def get_expected_value(self) -> Any:
        return "GRS"

    def get_expected_values(self) -> List[Any]:
        return ["GRS", "RAGRS", "GZRS", "RAGZRS"]


check = StorageAccountsUseReplication()



from __future__ import annotations

import logging
import platform
import stat
from abc import abstractmethod, ABC
from pathlib import Path
from typing import Dict, Any, List, TYPE_CHECKING
from datetime import datetime, timedelta

from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.util.data_structures_utils import merge_dicts
from checkov.common.util.http_utils import get_default_post_headers, request_wrapper, aiohttp_client_session_wrapper

if TYPE_CHECKING:
    from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration


class TwistcliIntegration(ABC):
    vulnerabilities_base_path = "/api/v1/vulnerabilities"  # noqa: CCE003  # a static attribute

    def get_bc_api_key(self) -> str:
        return bc_integration.get_auth_token()

    def get_proxy_address(self) -> str:
        return f"{bc_integration.api_url}{self.vulnerabilities_base_path}/docker-images/twistcli/proxy"

    def download_twistcli(self, cli_file_name: Path) -> bool:
        # backwards compatibility, should be removed in a later stage
        try:
            cli_file_name_path = cli_file_name if isinstance(cli_file_name, Path) else Path(cli_file_name)
            os_type = platform.system().lower()
            response = request_wrapper("GET",
                                       f"{bc_integration.api_url}{self.vulnerabilities_base_path}/twistcli?os={os_type}",
                                       headers=bc_integration.get_default_headers("GET"),
                                       should_call_raise_for_status=True)

            cli_file_name_path.write_bytes(response.content)
            cli_file_name_path.chmod(cli_file_name_path.stat().st_mode | stat.S_IEXEC)
            logging.debug("twistcli downloaded and has execute permission")
            return True
        except Exception:
            logging.debug(
                "Unexpected failure happened during downloading twistcli. details are below.\n"
                "scanning is terminating. please try again. if it is repeated, please report.\n", exc_info=True)
            return False

    async def report_results_async(
        self,
        twistcli_scan_result: Dict[str, Any],
        bc_platform_integration: BcPlatformIntegration,
        bc_api_key: str,
        file_path: Path,
        **kwargs: Any,
    ) -> int:
        logging.info(f"Start to send report for package file {file_path}")

        if not bc_platform_integration.bc_source:
            logging.error("Source was not set")
            return 1

        payload = self.create_report(
            twistcli_scan_result=twistcli_scan_result,
            bc_platform_integration=bc_platform_integration,
            file_path=file_path,
            **kwargs,
        )
        headers = merge_dicts(
            get_default_post_headers(bc_platform_integration.bc_source, bc_platform_integration.bc_source_version),
            {"Authorization": bc_api_key},
        )
        url = f"{bc_platform_integration.api_url}{self.vulnerabilities_base_path}/results"

        logging.info(f"[twistcli](report_results_async) - reporting results to the server for the file \'{file_path}\'")
        status: int = await aiohttp_client_session_wrapper(url, headers, payload)

        if status == 1:
            logging.error(f"[twistcli](report_results_async) - Failed to send report for package file {file_path}"
                          f"\nerror message appears above")
        return status

    @abstractmethod
    def create_report(
        self,
        twistcli_scan_result: dict[str, Any],
        bc_platform_integration: BcPlatformIntegration,
        file_path: Path,
        **kwargs: Any,
    ) -> dict[str, Any]:
        pass

    @staticmethod
    def get_vulnerabilities_for_report(scan_results: Dict[str, Any]) -> List[Dict[str, Any]]:
        return [
            {
                "cveId": vul.get("id"),
                "status": vul.get("status", "open"),
                "severity": vul.get("severity"),
                "packageName": vul.get("packageName"),
                "packageVersion": vul.get("packageVersion"),
                "link": vul.get("link"),
                "cvss": vul.get("cvss"),
                "vector": vul.get("vector"),
                "description": vul.get("description"),
                "riskFactors": vul.get("riskFactors"),
                "publishedDate": vul.get("publishedDate") or
                                 (datetime.now() - timedelta(days=vul.get("publishedDays", 0))).isoformat(),
            }
            for vul in scan_results.get("vulnerabilities") or []
        ]

    @staticmethod
    def get_packages_for_report(scan_results: Dict[str, Any]) -> List[Dict[str, Any]]:
        return [
            {
                "type": package.get("type"),
                "name": package.get("name"),
                "version": package.get("version"),
                "licenses": package.get("licenses") or [],
            }
            for package in scan_results.get("packages") or []
            if package.get("version") and package.get("name") and package.get("type")
        ]



from __future__ import annotations

import json
import logging
import os
from collections import defaultdict
from functools import partial
from pathlib import Path
from typing import List, Optional, Union, Any, Dict, overload, TypedDict, cast

import checkov.terraform.graph_builder.foreach.consts
from checkov.common.graph.graph_builder import Edge
from checkov.common.graph.graph_builder import reserved_attribute_names
from checkov.common.graph.graph_builder.graph_components.attribute_names import CustomAttributes
from checkov.common.graph.graph_builder.local_graph import LocalGraph
from checkov.common.graph.graph_builder.utils import calculate_hash, join_trimmed_strings, filter_sub_keys
from checkov.common.runners.base_runner import strtobool
from checkov.common.typing import TFDefinitionKeyType
from checkov.common.util.data_structures_utils import pickle_deepcopy
from checkov.common.util.type_forcers import force_int
from checkov.terraform.graph_builder.foreach.builder import ForeachBuilder
from checkov.terraform.graph_builder.variable_rendering.vertex_reference import TerraformVertexReference
from checkov.terraform.modules.module_objects import TFModule, TFDefinitionKey
from checkov.terraform.context_parsers.registry import parser_registry
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.graph_components.blocks import TerraformBlock
from checkov.terraform.graph_builder.graph_components.generic_resource_encryption import ENCRYPTION_BY_RESOURCE_TYPE
from checkov.terraform.graph_builder.graph_components.module import Module
from checkov.terraform.graph_builder.utils import (
    get_attribute_is_leaf,
    get_referenced_vertices_in_value,
    attribute_has_nested_attributes,
    remove_index_pattern_from_str,
    join_double_quote_surrounded_dot_split, )
from checkov.terraform.graph_builder.foreach.utils import get_terraform_foreach_or_count_key, \
    get_sanitized_terraform_resource_id
from checkov.terraform.graph_builder.utils import is_local_path
from checkov.terraform.graph_builder.variable_rendering.renderer import TerraformVariableRenderer, \
    LEFT_BRACKET_WITH_QUOTATION, RIGHT_BRACKET_WITH_QUOTATION, LEFT_BRACKET, RIGHT_BRACKET, DOLLAR_PREFIX, RIGHT_CURLY, LEFT_CURLY
from checkov.common.util.consts import RESOLVED_MODULE_ENTRY_NAME

MODULE_RESERVED_ATTRIBUTES = ("source", "version")
CROSS_VARIABLE_EDGE_PREFIX = '[cross-variable] '
S3_BUCKET_RESOURCE_NAME = "aws_s3_bucket"
S3_BUCKET_REFERENCE_ATTRIBUTE = "bucket"


class Undetermined(TypedDict):
    module_vertex_id: int
    attribute_name: str
    variable_vertex_id: int


class S3ConnectedResources(TypedDict):
    bucket_resource_index: int | None
    referenced_vertices: List[Edge]


class TerraformLocalGraph(LocalGraph[TerraformBlock]):
    def __init__(self, module: Module) -> None:
        super().__init__()
        self.vertices: list[TerraformBlock] = []
        self.module = module
        self.map_path_to_module: Dict[str, List[int]] = {}
        self.relative_paths_cache: dict[tuple[str, str], str] = {}
        self.abspath_cache: Dict[str, str] = {}
        self.dirname_cache: Dict[str, str] = {}
        self.vertices_by_module_dependency_by_name: Dict[TFModule | None, Dict[str, Dict[str, List[int]]]] = defaultdict(partial(defaultdict, partial(defaultdict, list)))
        self.vertices_by_module_dependency: Dict[TFModule | None, Dict[str, List[int]]] = defaultdict(partial(defaultdict, list))
        self.enable_foreach_handling = strtobool(os.getenv('CHECKOV_ENABLE_FOREACH_HANDLING', 'True'))
        self.enable_modules_foreach_handling = strtobool(os.getenv('CHECKOV_ENABLE_MODULES_FOREACH_HANDLING', 'True'))
        self.enable_datas_foreach_handling = strtobool(os.getenv('CHECKOV_ENABLE_DATAS_FOREACH_HANDLING', 'False'))
        self.foreach_blocks: Dict[str, List[int]] = {BlockType.RESOURCE: [], BlockType.MODULE: [], BlockType.DATA: []}

        # Important for foreach performance, see issue https://github.com/bridgecrewio/checkov/issues/6068
        self._vertex_path_to_realpath_cache: Dict[str, str] = {}

    def build_graph(self, render_variables: bool) -> None:
        self._create_vertices()
        logging.info(f"[TerraformLocalGraph] created {len(self.vertices)} vertices")
        self._build_edges()
        logging.info(f"[TerraformLocalGraph] created {len(self.edges)} edges")
        if (self.enable_foreach_handling or self.enable_modules_foreach_handling) \
                and (self.foreach_blocks[BlockType.RESOURCE] or self.foreach_blocks[BlockType.MODULE] or self.foreach_blocks[BlockType.DATA]):
            try:
                logging.info('[TerraformLocalGraph] start handling foreach')
                foreach_builder = ForeachBuilder(self)
                foreach_builder.handle(self.foreach_blocks)
                self._arrange_graph_data()
                self._build_edges()
                logging.info(f"[TerraformLocalGraph] finished handling foreach values with {len(self.vertices)} vertices and {len(self.edges)} edges")
            except Exception as e:
                logging.info(f'Failed to process foreach handling, error: {str(e)}')

        self.calculate_encryption_attribute(ENCRYPTION_BY_RESOURCE_TYPE)
        self._connect_module_provider()
        if render_variables:
            logging.info(f"Rendering variables, graph has {len(self.vertices)} vertices and {len(self.edges)} edges")
            renderer = TerraformVariableRenderer(self)
            renderer.render_variables_from_local_graph()
            self.update_vertices_fields()
            if strtobool(os.getenv("CHECKOV_EXPERIMENTAL_CROSS_VARIABLE_EDGES", "True")):
                # experimental flag on building cross variable edges for terraform graph
                logging.info("Building cross variable edges")
                edges_count = len(self.edges)
                self._build_cross_variable_edges()
                logging.info(f"Found {len(self.edges) - edges_count} cross variable edges")
            # building S3 edges by name for terraform graph
            logging.info("Building S3 edges name references")
            edges_count = len(self.edges)
            self._build_s3_name_reference_edges()
            logging.info(f"Found {len(self.edges) - edges_count} S3 name references edges")
        else:
            self.update_vertices_fields()

    def update_vertices_fields(self) -> None:
        # Important to keep those 2 functions together, as the first affects the calculation of the second
        self._update_vertices_breadcrumbs_and_module_connections()
        self._update_nested_modules_address()
        try:
            self._add_provider_attr_to_resources()
        except Exception as e:
            logging.info(f'Failed to add provider attributes to resources, error: {str(e)}')

    def _create_vertices(self) -> None:
        logging.info("Creating vertices")
        self.vertices = [None] * len(self.module.blocks)  # type:ignore[list-item]  # are correctly set in the next lines
        for i, block in enumerate(self.module.blocks):
            self.vertices[i] = block
            self._add_block_data_to_graph(i, block)
            if self.enable_foreach_handling and (
                    checkov.terraform.graph_builder.foreach.consts.FOREACH_STRING in block.attributes or checkov.terraform.graph_builder.foreach.consts.COUNT_STRING in block.attributes) \
                    and block.block_type in (BlockType.MODULE, BlockType.RESOURCE, BlockType.DATA):
                self.foreach_blocks[block.block_type].append(i)

    def _add_block_data_to_graph(self, idx: int, block: TerraformBlock) -> None:
        self.vertices_by_block_type[block.block_type].append(idx)
        self.vertices_block_name_map[block.block_type][block.name].append(idx)

        if block.block_type == BlockType.MODULE:
            # map between file paths and module vertices indexes from that file
            self.map_path_to_module.setdefault(block.path, []).append(idx)

        self.vertices_by_module_dependency[block.source_module_object][block.block_type].append(idx)
        self.vertices_by_module_dependency_by_name[block.source_module_object][block.block_type][block.name].append(idx)

        self.in_edges[idx] = []
        self.out_edges[idx] = []

    def _add_provider_attr_to_resources(self) -> None:
        """
            Assign provider attributes to resource vertices in the Terraform configuration.

            This function iterates through all vertices, and for each vertex that is of the
            `RESOURCE` block type, it attempts to determine and assign the appropriate
            provider based on the vertex's attributes and the module's temporary Terraform
            definitions.

            The function performs the following steps:
            1. Iterates over each vertex in `self.vertices`.
            2. Checks if the vertex's `block_type` is `RESOURCE`.
            3. Determines the path for the Terraform definition.
            4. Checks for the presence of a provider in the vertex's attributes.
            5. If no provider is found, checks in the module's temporary Terraform definition.
            6. If still no provider is found, iterates through nested modules to find a provider.
            7. Assigns the provider fields to the vertex once a provider is determined.
        """
        for vertex in self.vertices:
            if vertex.block_type == BlockType.RESOURCE:
                path_for_tf_definition = TFDefinitionKey(file_path=vertex.path, tf_source_modules=vertex.source_module_object)
                if BlockType.PROVIDER in vertex.attributes:
                    provider = self._get_the_default_provider(vertex.attributes[BlockType.PROVIDER], path_for_tf_definition)
                    self._assign_provider_fields(vertex, provider)
                elif BlockType.PROVIDER in self.module.temp_tf_definition.get(path_for_tf_definition, {}):
                    provider_name = self._get_the_default_provider(self.module.temp_tf_definition.get(path_for_tf_definition, {}).get(BlockType.PROVIDER, []), path_for_tf_definition, is_same_file=True)
                    v_name = vertex.name.split('.')
                    provider_name = vertex.config[v_name[0]][v_name[1]][CustomAttributes.TF_RESOURCE_ADDRESS].replace(vertex.name, provider_name)
                    self._assign_provider_fields(vertex, provider_name)
                else:
                    while path_for_tf_definition.tf_source_modules:
                        if self.module.temp_tf_definition and path_for_tf_definition.tf_source_modules and BlockType.PROVIDER in self.module.temp_tf_definition.get(path_for_tf_definition, ''):
                            module = [m for m in self.module.temp_tf_definition.get(path_for_tf_definition).get(BlockType.MODULE) if list(m.keys())[0] == vertex.source_module_object.name]  # type:ignore
                            provider_name = self._get_the_default_provider(self.module.temp_tf_definition.get(TFDefinitionKey(path_for_tf_definition.tf_source_modules.path), {}).get(BlockType.PROVIDER, []), path_for_tf_definition, module=module)
                            self._assign_provider_fields(vertex, provider_name)
                            break
                        elif path_for_tf_definition.tf_source_modules and BlockType.PROVIDER in self.module.temp_tf_definition.get(TFDefinitionKey(path_for_tf_definition.tf_source_modules.path), {}):
                            if vertex.source_module_object and vertex.source_module_object.nested_tf_module:
                                module = [m for m in self.module.temp_tf_definition.get(TFDefinitionKey(path_for_tf_definition.tf_source_modules.path)).get(BlockType.MODULE) if list(m.keys())[0] == vertex.source_module_object.nested_tf_module.name]  # type:ignore
                                provider_name = self._get_the_default_provider(self.module.temp_tf_definition.get(TFDefinitionKey(path_for_tf_definition.tf_source_modules.path), {}).get(BlockType.PROVIDER, []), path_for_tf_definition, module=module)
                                self._assign_provider_fields(vertex, provider_name)
                            else:
                                module = [m for m in self.module.temp_tf_definition.get(TFDefinitionKey(path_for_tf_definition.tf_source_modules.path)).get(BlockType.MODULE) if list(m.keys())[0] == vertex.source_module_object.name]  # type:ignore
                                provider_name = self._get_the_default_provider(self.module.temp_tf_definition.get(TFDefinitionKey(path_for_tf_definition.tf_source_modules.path), {}).get(BlockType.PROVIDER, []), path_for_tf_definition, module=module)
                                self._assign_provider_fields(vertex, provider_name)
                            break
                        else:
                            path_for_tf_definition = TFDefinitionKey(tf_source_modules=path_for_tf_definition.tf_source_modules.nested_tf_module, file_path=path_for_tf_definition.tf_source_modules.path)

    @staticmethod
    def _assign_provider_fields(vertex: TerraformBlock, provider_name: str) -> None:
        vertex.attributes[CustomAttributes.PROVIDER_ADDRESS] = provider_name
        v_name = vertex.name.split('.')
        vertex.config[v_name[0]][v_name[1]][CustomAttributes.PROVIDER_ADDRESS] = provider_name

    def _get_provider_address(self, path_for_tf_definition: TFDefinitionKeyType) -> list[int] | None:
        if isinstance(path_for_tf_definition, TFDefinitionKey):
            return self.vertices_by_module_dependency[path_for_tf_definition.tf_source_modules].get(BlockType.PROVIDER)
        return None

    def _get_the_default_provider(
            self,
            providers: list[dict[str, dict[str, Any]] | str],
            path_for_tf_definition: TFDefinitionKeyType,
            module: Optional[list[dict[str, Any]]] = None,
            is_same_file: bool = False
    ) -> str:
        provider_address = self._get_provider_address(path_for_tf_definition)
        if module and len(module) > 0:
            module_providers = module[0][list(module[0].keys())[0]].get('providers', [''])[0]
            if module_providers:
                for _, m_alias in module_providers.items():
                    if not provider_address:
                        return cast(str, module_providers[list(module_providers.keys())[0]].replace(DOLLAR_PREFIX, "").replace(LEFT_CURLY, "").replace(RIGHT_CURLY, ""))
                    else:
                        for p_address in provider_address:
                            if m_alias.replace(DOLLAR_PREFIX, "").replace(LEFT_CURLY, "").replace(RIGHT_CURLY, "") == self.vertices[p_address].name:
                                return cast(str, self.vertices[p_address].config[list(self.vertices[p_address].config)[0]].get(CustomAttributes.TF_RESOURCE_ADDRESS))

        if isinstance(providers[0], str):
            for provider in providers:
                if provider_address:
                    for address in provider_address:
                        if self.vertices[address].name == provider:
                            return cast(str, self.vertices[address].config[list(self.vertices[address].config)[0]].get(CustomAttributes.TF_RESOURCE_ADDRESS))
        else:
            for provider in providers:
                provider_name = list(provider.keys())[0]  # type:ignore
                if 'alias' not in provider[provider_name]:
                    if provider_address and not is_same_file:
                        for p_address in provider_address:
                            if self.vertices[p_address].name == provider_name:
                                return cast(str, self.vertices[p_address].config[list(self.vertices[p_address].config.keys())[0]][CustomAttributes.TF_RESOURCE_ADDRESS])
                    return f'{provider_name}.default'
        return ''

    def _arrange_graph_data(self) -> None:
        # reset all the relevant data
        self.vertices_by_block_type = defaultdict(list)
        self.vertices_block_name_map = defaultdict(partial(defaultdict, list))
        self.map_path_to_module = {}
        self.vertices_by_module_dependency = defaultdict(partial(defaultdict, list))
        self.vertices_by_module_dependency_by_name = defaultdict(partial(defaultdict, partial(defaultdict, list)))
        self.edges = []
        for i in range(len(self.vertices)):
            self.out_edges[i] = []
            self.in_edges[i] = []

        for i, block in enumerate(self.vertices):
            self._add_block_data_to_graph(i, block)

    def _get_aliases(self) -> Dict[str, Dict[str, str]]:
        """
        :return aliases: map between alias names that are found inside the blocks and the block type their aliased to.
        """
        return {
            vertex.name: {CustomAttributes.BLOCK_TYPE: vertex.block_type}
            for vertex in self.vertices
            if "alias" in vertex.attributes
        }

    def get_module_vertices_mapping(self) -> None:
        """
        For each vertex, if it's originated in a module import, add to the vertex the index of the
        matching module vertex as 'source_module'
        """
        for vertex in self.vertices:
            if not vertex.source_module_object:
                continue
            for idx in self.vertices_by_block_type[BlockType.MODULE]:
                if vertex.source_module_object.name != self.vertices[idx].name:
                    continue
                if vertex.source_module_object.path != self.vertices[idx].path:
                    continue
                if vertex.source_module_object.nested_tf_module != self.vertices[idx].source_module_object:
                    continue
                if vertex.source_module_object.foreach_idx != self.vertices[idx].for_each_index:
                    continue
                vertex.source_module.add(idx)
                break
        return

    def _build_edges(self) -> None:
        logging.info("Creating edges")
        self.get_module_vertices_mapping()
        aliases = self._get_aliases()
        resources_types = self.get_resources_types_in_graph()
        for origin_node_index, vertex in enumerate(self.vertices):
            self._build_edges_for_vertex(origin_node_index, vertex, aliases, resources_types)

    def _build_edges_for_vertex(self, origin_node_index: int, vertex: TerraformBlock, aliases: Dict[str, Dict[str, str]],
                                resources_types: List[str], cross_variable_edges: bool = False,
                                referenced_modules: Optional[List[Dict[str, Any]]] = None) -> None:

        attribute_is_leaf = get_attribute_is_leaf(vertex)
        for attribute_key, attribute_value in vertex.attributes.items():
            if attribute_key in reserved_attribute_names or attribute_has_nested_attributes(
                    attribute_key, vertex.attributes, attribute_is_leaf
            ):
                continue
            referenced_vertices = get_referenced_vertices_in_value(
                value=attribute_value,
                aliases=aliases,
                resources_types=resources_types,
            )
            for vertex_reference in referenced_vertices:
                # for certain blocks such as data and resource, the block name is composed of several parts.
                # the purpose of the loop is to avoid not finding the node if the name has several parts
                sub_values = [remove_index_pattern_from_str(sub_value) for sub_value in vertex_reference.sub_parts]
                for i in range(len(sub_values)):
                    reference_name = join_trimmed_strings(char_to_join=".", str_lst=sub_values, num_to_trim=i)
                    source_module_object = vertex.source_module_object
                    if referenced_modules is not None:
                        for module in referenced_modules:
                            referenced_module_idx = module.get("idx")
                            referenced_module_path = module.get("path")
                            if referenced_module_path is None:
                                dest_node_index = -1
                            else:
                                dest_node_index = self._find_vertex_index_relative_to_path(
                                    vertex_reference.block_type, reference_name, referenced_module_path,
                                    referenced_module_idx,
                                    source_module_object=source_module_object
                                )
                            self._create_edge_from_reference(attribute_key, origin_node_index, dest_node_index,
                                                             sub_values, vertex_reference, cross_variable_edges)
                    if vertex.source_module_object:
                        dest_node_index = self._find_vertex_index_relative_to_path(
                            vertex_reference.block_type, reference_name, vertex.path,
                            source_module_object=source_module_object
                        )
                        if dest_node_index == -1:
                            dest_node_index = self._find_vertex_index_relative_to_path(
                                vertex_reference.block_type, reference_name, vertex.path,
                                source_module_object=source_module_object
                            )
                    else:
                        dest_node_index = self._find_vertex_index_relative_to_path(
                            vertex_reference.block_type, reference_name, vertex.path,
                            source_module_object=source_module_object, origin_vertex_index=origin_node_index
                        )
                    if dest_node_index > -1 and origin_node_index > -1:
                        self._create_edge_from_reference(attribute_key, origin_node_index, dest_node_index, sub_values,
                                                         vertex_reference, cross_variable_edges)
                        break

        if vertex.block_type == BlockType.MODULE and vertex.attributes.get('source') \
                and isinstance(vertex.attributes['source'][0], str):
            dest_module_path = self._get_dest_module_path(
                curr_module_dir=self.get_dirname(vertex.path),
                dest_module_source=vertex.attributes["source"][0],
                dest_module_version=vertex.attributes.get("version", ["latest"])[0]
            )
            target_variables = self._get_target_variables(vertex, dest_module_path)
            for attribute in vertex.attributes.keys():
                if attribute in MODULE_RESERVED_ATTRIBUTES:
                    continue
                target_variable = next((v for v in target_variables if self.vertices[v].name == attribute), None)
                if target_variable is not None:
                    self.create_edge(target_variable, origin_node_index, "default", cross_variable_edges)
        elif vertex.block_type == BlockType.TF_VARIABLE:
            # Assuming the tfvars file is in the same directory as the variables file (best practice)
            target_variable = 0
            for index in self.vertices_block_name_map.get(BlockType.VARIABLE, {}).get(vertex.name, []):
                if self.get_dirname(self.vertices[index].path) == self.get_dirname(vertex.path):
                    target_variable = index
                    break
            if target_variable:
                self.create_edge(target_variable, origin_node_index, "default", cross_variable_edges)

    def _create_edge_from_reference(self, attribute_key: Any, origin_node_index: int, dest_node_index: int,
                                    sub_values: List[Any], vertex_reference: TerraformVertexReference,
                                    cross_variable_edges: bool) -> None:
        if dest_node_index > -1 and origin_node_index > -1:
            if vertex_reference.block_type == BlockType.MODULE:
                try:
                    self._connect_module(
                        sub_values, attribute_key, self.vertices[dest_node_index],
                        origin_node_index,
                        cross_variable_edges
                    )
                except Exception:
                    logging.warning(
                        f"Module {self.vertices[dest_node_index]} does not have source attribute, skipping"
                    )
            else:
                self.create_edge(origin_node_index, dest_node_index, attribute_key,
                                 cross_variable_edges)

    def _get_target_variables(self, vertex: TerraformBlock, dest_module_path: str) -> list[int]:
        target_path = get_vertex_as_tf_module(vertex)
        return [
            index
            for index in self.vertices_by_module_dependency.get(target_path, {}).get(BlockType.VARIABLE, [])
            if self.get_dirname(self.vertices[index].path) == dest_module_path
        ]

    def _connect_module_provider(self) -> None:
        for origin_node_index, referenced_vertices in self.out_edges.items():
            if not referenced_vertices:
                continue
            vertex = self.vertices[origin_node_index]
            # if we have an edge of module->provider we need to connect that modules' resources to the provider
            if vertex.block_type == BlockType.MODULE:
                try:
                    tf_def = vertex.config.get(vertex.name, {}).get(RESOLVED_MODULE_ENTRY_NAME)
                    if tf_def and isinstance(tf_def, list):
                        if isinstance(tf_def[0], str):
                            definition = json.loads(tf_def[0])
                            tf_module = TFDefinitionKey.from_json(definition).tf_source_modules
                        else:
                            tf_module = tf_def[0].tf_source_modules
                        # get all resources connected to module
                        resources = self.vertices_by_module_dependency[tf_module].get("resource")
                        if resources:
                            # search for provider vertices in the referenced vertices
                            for e in referenced_vertices:
                                if self.vertices[e.dest].block_type == BlockType.PROVIDER:
                                    for resource in resources:
                                        self.create_edge(resource, e.dest, e.label)
                except Exception as e:
                    logging.warning(f"Failed in connecting module resources to provider due to {e}")

    def _build_cross_variable_edges(self) -> None:
        aliases = self._get_aliases()
        resources_types = self.get_resources_types_in_graph()
        for origin_node_index, referenced_vertices in self.out_edges.items():
            vertex = self.vertices[origin_node_index]
            if vertex.block_type == BlockType.RESOURCE and \
                    any(self.vertices[e.dest].block_type != BlockType.RESOURCE for e in referenced_vertices):
                modules = vertex.breadcrumbs.get(CustomAttributes.SOURCE_MODULE, [])
                self._build_edges_for_vertex(origin_node_index, vertex, aliases, resources_types, True, modules)

    def _build_s3_name_reference_edges(self) -> None:
        # Supporting reference by name of S3 bucket
        resources_types = self.get_resources_types_in_graph()
        if S3_BUCKET_RESOURCE_NAME not in resources_types:
            return
        # Find all the edges leading to S3 bucket and their references
        s3_buckets_mapping: Dict[int, S3ConnectedResources] = {}
        for origin_node_index, referenced_vertices in self.out_edges.items():
            vertex = self.vertices[origin_node_index]
            if vertex.block_type != BlockType.RESOURCE:
                continue
            for referenced_vertice in referenced_vertices:
                if referenced_vertice.label == S3_BUCKET_REFERENCE_ATTRIBUTE:
                    current = s3_buckets_mapping.get(referenced_vertice.dest, {"bucket_resource_index": None, "referenced_vertices": list()})
                    if vertex.id.startswith(f"{S3_BUCKET_RESOURCE_NAME}."):
                        current["bucket_resource_index"] = origin_node_index
                    else:
                        current["referenced_vertices"].append(referenced_vertice)
                    s3_buckets_mapping[referenced_vertice.dest] = current

        # Create new edges of the found connections
        for destination, mapping in s3_buckets_mapping.items():
            if self.vertices[destination].block_type in [BlockType.VARIABLE, BlockType.LOCALS]:
                if mapping["bucket_resource_index"] is None:
                    continue
                for reference_vertex in mapping["referenced_vertices"]:
                    self.create_edge(mapping["bucket_resource_index"], reference_vertex.origin, S3_BUCKET_REFERENCE_ATTRIBUTE, True)

    def create_edge(self, origin_vertex_index: int, dest_vertex_index: int, label: str,
                    cross_variable_edges: bool = False) -> bool:
        if origin_vertex_index == dest_vertex_index:
            return False
        edge = Edge(origin_vertex_index, dest_vertex_index, label)
        if cross_variable_edges:
            if self.vertices[dest_vertex_index].block_type != BlockType.RESOURCE or \
                    self.vertices[origin_vertex_index].block_type != BlockType.RESOURCE:
                return False
            if edge in self.out_edges[origin_vertex_index]:
                return False
            edge.label = CROSS_VARIABLE_EDGE_PREFIX + edge.label
            if edge in self.out_edges[origin_vertex_index]:
                return False
        self.edges.append(edge)
        self.out_edges[origin_vertex_index].append(edge)
        self.in_edges[dest_vertex_index].append(edge)
        return True

    def _connect_module(
        self, sub_values: List[str], attribute_key: str, module_node: TerraformBlock, origin_node_index: int, cross_variable_edges: bool = False
    ) -> None:
        """
        :param sub_values: list of sub values of the attribute value.
                            example: given 'instance_type = module.child.myoutput',
                                        then attribute_key = instance_type, sub_values = ['child', 'myoutput']
        :param attribute_key: the name of the attribute that has module block as value
        :param module_node: the graph node of the module

        The function receives a node of a block of type BlockType.Module, and finds all the nodes of blocks that belong to this
        module, and creates edges between them.
        """
        curr_module_dir = self.get_dirname(module_node.path)
        dest_module_source = module_node.attributes["source"][0]
        dest_module_version = module_node.attributes.get("version", ["latest"])[0]
        dest_module_path = self._get_dest_module_path(
            curr_module_dir=curr_module_dir,
            dest_module_source=dest_module_source,
            dest_module_version=dest_module_version
        )

        if len(sub_values) > 1:
            block_name_in_other_module = sub_values[1]
            output_blocks_with_name = self.vertices_block_name_map.get(BlockType.OUTPUT, {}).get(
                block_name_in_other_module, []
            )
            for vertex_index in output_blocks_with_name:
                vertex = self.vertices[vertex_index]
                if self._should_add_edge(vertex, dest_module_path, module_node):
                    added_edge = self.create_edge(origin_node_index, vertex_index, attribute_key, cross_variable_edges)
                    if added_edge:
                        self.vertices[origin_node_index].add_module_connection(attribute_key, vertex_index)
                    break

    def _get_dest_module_path(self, curr_module_dir: str, dest_module_source: str, dest_module_version: str) -> str:
        """
        :param curr_module_dir: current source directory
        :param dest_module_source: the value of module.source
        :return: the real path in the local file system of the dest module
        """
        relative_path_key = (curr_module_dir, dest_module_source)
        if relative_path_key in self.relative_paths_cache:
            return self.relative_paths_cache[relative_path_key]
        elif is_local_path(curr_module_dir, dest_module_source):
            self.relative_paths_cache[relative_path_key] = os.path.abspath(Path(curr_module_dir) / dest_module_source)
            return self.relative_paths_cache[relative_path_key]
        elif (dest_module_source, dest_module_version) in self.module.external_modules_source_map:
            return self.module.external_modules_source_map[(dest_module_source, dest_module_version)]

        # this happens, when we have external modules, which weren't downloaded
        return ""

    def _find_vertex_index_relative_to_path(
        self,
        block_type: str,
        name: str,
        block_path: str,
        relative_module_idx: Optional[int] = None,
        source_module_object: Optional[TFModule] = None,
        origin_vertex_index: Optional[int] = None,
    ) -> int:
        relative_vertices: list[int] = []
        if relative_module_idx is None:
            module_dependency_by_name_key = source_module_object
        else:
            vertex = self.vertices[relative_module_idx]
            module_dependency_by_name_key = vertex.source_module_object

        # important to use this specific map for big graph performance
        possible_vertices = self._get_possible_vertices(module_dependency_by_name_key, block_type, name)
        for vertex_index in possible_vertices:
            vertex = self.vertices[vertex_index]
            if self.get_dirname(vertex.path) == self.get_dirname(block_path):
                relative_vertices.append(vertex_index)

        if len(relative_vertices) == 1:
            relative_vertex = relative_vertices[0]
        else:
            relative_vertex = self._find_vertex_with_best_match(relative_vertices, block_path, origin_vertex_index)
        return relative_vertex

    def _get_possible_vertices(self, module_dependency_by_name_key: TFModule | None, block_type: str, name: str) -> list[int]:
        possible_vertices = self.vertices_by_module_dependency_by_name.get(module_dependency_by_name_key, {}).get(block_type, {}).get(name, [])
        if possible_vertices:
            return possible_vertices
        return self.vertices_by_module_dependency_by_name.get(module_dependency_by_name_key, {}).get(block_type, {}).get(name.replace(LEFT_BRACKET_WITH_QUOTATION, LEFT_BRACKET).replace(RIGHT_BRACKET_WITH_QUOTATION, RIGHT_BRACKET), [])

    def _find_vertex_with_best_match(self, relevant_vertices_indexes: List[int], origin_path: str,
                                     origin_vertex_index: Optional[int] = None) -> int:
        vertex_index_with_longest_common_prefix = -1
        longest_common_prefix = ""
        vertices_with_longest_common_prefix = []
        origin_real_path = os.path.realpath(origin_path)
        for vertex_index in relevant_vertices_indexes:
            vertex = self.vertices[vertex_index]
            if vertex.path in self._vertex_path_to_realpath_cache:
                # Using cache to make sure performance stays stable
                vertex_realpath = self._vertex_path_to_realpath_cache[vertex.path]
            else:
                vertex_realpath = os.path.realpath(vertex.path)
                self._vertex_path_to_realpath_cache[vertex.path] = vertex_realpath
            common_prefix = os.path.commonpath([vertex_realpath, origin_real_path])

            # checks if module name is same for dest and origin vertex.
            if origin_vertex_index is not None:
                vertex_module_name = vertex.attributes.get(CustomAttributes.TF_RESOURCE_ADDRESS, '')
                origin_module_name = self.vertices[origin_vertex_index].attributes.get(CustomAttributes.TF_RESOURCE_ADDRESS, '')
                if vertex_module_name.startswith(BlockType.MODULE) and origin_module_name.startswith(BlockType.MODULE):
                    split_module_name = vertex_module_name.split('.')[1]
                    if origin_module_name.startswith(f'{BlockType.MODULE}.{split_module_name}'):
                        common_prefix = f"{common_prefix} {BlockType.MODULE}.{split_module_name}"

            if len(common_prefix) > len(longest_common_prefix):
                vertex_index_with_longest_common_prefix = vertex_index
                longest_common_prefix = common_prefix
                vertices_with_longest_common_prefix = [(vertex_index, vertex)]
            elif len(common_prefix) == len(longest_common_prefix):
                vertices_with_longest_common_prefix.append((vertex_index, vertex))

        if len(vertices_with_longest_common_prefix) <= 1:
            return vertex_index_with_longest_common_prefix

        # Try to compare based on foreach attributes if we have more than 1 vertex in the list
        if origin_vertex_index is not None:
            return self._find_best_match_based_on_foreach_key(origin_vertex_index, vertices_with_longest_common_prefix,
                                                              vertex_index_with_longest_common_prefix)
        return vertex_index_with_longest_common_prefix

    def _find_best_match_based_on_foreach_key(
            self,
            origin_vertex_index: int,
            vertices_with_longest_common_prefix: list[tuple[int, TerraformBlock]],
            vertex_index_with_longest_common_prefix: int
    ) -> int:
        origin_vertex = self.vertices[origin_vertex_index]
        for vertex_index, vertex in vertices_with_longest_common_prefix:
            vertex_address = vertex.attributes.get(CustomAttributes.TF_RESOURCE_ADDRESS, '')
            vertex_foreach_value = vertex.for_each_index
            origin_address = origin_vertex.attributes.get(CustomAttributes.TF_RESOURCE_ADDRESS, '')
            origin_foreach_value = origin_vertex.for_each_index
            if origin_foreach_value == vertex_foreach_value and origin_address != '' and \
                    get_terraform_foreach_or_count_key(origin_address) == \
                    get_terraform_foreach_or_count_key(vertex_address):
                return vertex_index

        return vertex_index_with_longest_common_prefix

    def get_vertices_hash_codes_to_attributes_map(self) -> Dict[str, Dict[str, Any]]:
        return {vertex.get_hash(): vertex.get_attribute_dict() for vertex in self.vertices}

    def order_edges_by_hash_codes(self) -> Dict[str, Edge]:
        edges = {}
        for edge in self.edges:
            edge_data = {
                "edge_label": edge.label,
                "from_vertex_hash": self.get_vertex_hash_by_index(vertex_index=edge.origin),
                "to_vertex_hash": self.get_vertex_hash_by_index(vertex_index=edge.dest),
            }
            edge_hash = calculate_hash(edge_data)
            edges[edge_hash] = edge
        return edges

    def get_vertex_hash_by_index(self, vertex_index: int) -> str:
        return self.vertex_hash_cache.setdefault(vertex_index, self.vertices[vertex_index].get_hash())

    def update_vertex_attribute(
        self,
        vertex_index: int,
        attribute_key: str,
        attribute_value: Any,
        change_origin_id: int | None,
        attribute_at_dest: Optional[Union[str, List[str]]],
        transform_step: bool = False,
    ) -> None:
        if change_origin_id is None:
            # no need to proceed further
            return

        previous_breadcrumbs = []
        attribute_at_dest = self.vertices[change_origin_id].find_attribute(attribute_at_dest)
        if attribute_at_dest:
            previous_breadcrumbs = self.vertices[change_origin_id].changed_attributes.get(attribute_at_dest, [])
        self.vertices[vertex_index].update_attribute(
            attribute_key, attribute_value, change_origin_id, previous_breadcrumbs, attribute_at_dest
        )

    def update_vertices_configs(self) -> None:
        for vertex in self.vertices:
            changed_attributes = list(vertex.changed_attributes.keys())
            changed_attributes = filter_sub_keys(changed_attributes)
            self.update_vertex_config(vertex, changed_attributes)

    @staticmethod
    def update_vertex_config(vertex: TerraformBlock, changed_attributes: Union[List[str], Dict[str, Any]], dynamic_blocks: bool = False) -> None:
        if not changed_attributes:
            # skip, if there is no change
            return

        vertex_name = vertex.name
        updated_config = pickle_deepcopy(vertex.config)
        if vertex.block_type == BlockType.PROVIDER:
            # provider blocks set the alias as a suffix to the name, ex. name: "aws.prod"
            vertex_name = vertex_name.split(".")[0]
        if vertex.block_type != BlockType.LOCALS:
            parts = vertex_name.split(".")
            start = 0
            end = 1
            while end <= len(parts):
                cur_key = ".".join(parts[start:end])
                if cur_key in updated_config:
                    updated_config = updated_config[cur_key]
                    start = end
                end += 1

        for changed_attribute in changed_attributes:
            new_value = vertex.attributes.get(changed_attribute, None)
            if new_value is not None:
                if vertex.block_type == BlockType.LOCALS:
                    changed_attribute = changed_attribute.replace(f"{vertex_name}.", "")
                updated_config = update_dictionary_attribute(updated_config, changed_attribute, new_value, dynamic_blocks)

        if len(changed_attributes) > 0:
            if vertex.block_type == BlockType.LOCALS:
                updated_local_config = updated_config.get(vertex_name)
                update_dictionary_attribute(vertex.config, vertex_name, updated_local_config, dynamic_blocks)
                return

            update_dictionary_attribute(vertex.config, vertex_name, updated_config, dynamic_blocks)

    def get_resources_types_in_graph(self) -> List[str]:
        return self.module.get_resources_types()

    def _update_vertices_breadcrumbs_and_module_connections(self) -> None:
        """
        The function processes each vertex's breadcrumbs:
        1. Get more data to each vertex in breadcrumb (name, path, hash and type)
        2. If a breadcrumb is originated in a different module, it will have 'module_connection'=True
        3. If a vertex has a 'source module' we will add a special breadcrumb for it
        """
        for vertex in self.vertices:
            for attribute_key, breadcrumbs_list in vertex.changed_attributes.items():
                hash_breadcrumbs = []
                for breadcrumb in breadcrumbs_list:
                    v = self.vertices[breadcrumb.vertex_id]
                    breadcrumb = v.get_export_data()
                    breadcrumb["module_connection"] = self._determine_if_module_connection(breadcrumbs_list, v)
                    hash_breadcrumbs.append(breadcrumb)
                vertex.breadcrumbs[attribute_key] = hash_breadcrumbs
            if len(vertex.source_module) == 1:
                v = vertex
                source_module_data = []
                while len(v.source_module) == 1:
                    idx = list(v.source_module)[0]
                    v = self.vertices[idx]
                    module_data = v.get_export_data()
                    module_data["idx"] = idx
                    if hasattr(vertex, "source_module_object"):
                        module_data["source_module_object"] = v.source_module_object
                    source_module_data.append(module_data)
                source_module_data.reverse()
                vertex.breadcrumbs[CustomAttributes.SOURCE_MODULE] = source_module_data

    @staticmethod
    def _determine_if_module_connection(breadcrumbs_list: List[int], vertex_in_breadcrumbs: TerraformBlock) -> bool:
        """
        :param breadcrumbs_list: list of vertex's breadcrumbs
        :param vertex_in_breadcrumbs: one of the vertices in the breadcrumb list
        :return: True if vertex_in_breadcrumbs has in its module_connections at least one of the vertices in breadcrumbs_list
        """
        if not vertex_in_breadcrumbs.module_connections:
            return False
        for connection_list in vertex_in_breadcrumbs.module_connections.values():
            if any(i in breadcrumbs_list for i in connection_list):
                return True
        return False

    def get_dirname(self, path: str) -> str:
        dir_name = self.dirname_cache.get(path)
        if not dir_name:
            dir_name = os.path.dirname(path)
            self.dirname_cache[path] = dir_name
        return dir_name

    def get_abspath(self, path: str) -> str:
        dir_name = self.abspath_cache.get(path)
        if not dir_name:
            dir_name = os.path.abspath(path)
            self.abspath_cache[path] = dir_name
        return dir_name

    def _update_nested_modules_address(self) -> None:
        for vertex in self.vertices:
            if vertex.attributes.get(CustomAttributes.TF_RESOURCE_ADDRESS) is not None:
                # Can happen for example in `tf_plan` files as the address already exists
                continue
            if vertex.block_type not in parser_registry.context_parsers:
                continue
            source_module = vertex.breadcrumbs.get(CustomAttributes.SOURCE_MODULE)

            address_prefix = ''
            if source_module:
                for module in source_module:
                    address_prefix += f"{module.get('type')}.{module.get('name')}."

            context_parser = parser_registry.context_parsers[vertex.block_type]
            entity_context_path = context_parser.get_entity_context_path(vertex.config)
            resource_id = '.'.join(entity_context_path) if entity_context_path else vertex.name
            address = f'{address_prefix}{resource_id}'
            vertex.attributes[CustomAttributes.TF_RESOURCE_ADDRESS] = address

            vertex_context = vertex.config
            definition_path = context_parser.get_entity_definition_path(vertex.config)
            for path in definition_path:
                vertex_context = vertex_context.get(path, vertex_context)
            vertex_context[CustomAttributes.TF_RESOURCE_ADDRESS] = address

    def _should_add_edge(self, vertex: TerraformBlock, dest_module_path: str, module_node: TerraformBlock) -> bool:
        if not vertex.source_module_object:
            return False

        return (self.get_dirname(vertex.path) == dest_module_path) and \
            (
                vertex.source_module_object == module_node.source_module_object  # The vertex is in the same file
                or self.get_abspath(vertex.source_module_object.path)
                == self.get_abspath(module_node.path)  # The vertex is in the correct dependency path)
        )


def to_list(data: Any) -> list[Any] | dict[str, Any]:
    if isinstance(data, list) and len(data) == 1 and (isinstance(data[0], str) or isinstance(data[0], int)):
        return data
    elif isinstance(data, list):
        return [to_list(x) for x in data]
    elif isinstance(data, dict):
        return {key: to_list(val) for key, val in data.items()}
    else:
        return [data]


@overload
def update_dictionary_attribute(
        config: dict[str, Any], key_to_update: str, new_value: Any, dynamic_blocks: bool = False
) -> dict[str, Any]:
    ...


@overload
def update_dictionary_attribute(
        config: list[Any], key_to_update: str, new_value: Any, dynamic_blocks: bool = False
) -> list[Any]:
    ...


def update_dictionary_attribute(
    config: Union[List[Any], Dict[str, Any]], key_to_update: str, new_value: Any, dynamic_blocks: bool = False
) -> Union[List[Any], Dict[str, Any]]:
    key_parts = key_to_update.split(".")
    if '"' in key_to_update:
        key_parts = join_double_quote_surrounded_dot_split(str_parts=key_parts)

    if isinstance(config, dict) and isinstance(key_parts, list):
        key = key_parts[0]
        inner_config = config.get(key)

        if inner_config is not None:
            if len(key_parts) == 1:
                if isinstance(inner_config, list) and not isinstance(new_value, list):
                    new_value = [new_value]
                config[key] = to_list(new_value) if dynamic_blocks else new_value
                return config
            else:
                config[key] = update_dictionary_attribute(
                    inner_config, ".".join(key_parts[1:]), new_value, dynamic_blocks=dynamic_blocks
                )
        else:
            for key in config:
                config[key] = update_dictionary_attribute(
                    config[key], key_to_update, new_value, dynamic_blocks=dynamic_blocks
                )
    if isinstance(config, list):
        return update_list_attribute(
            config=config,
            key_parts=key_parts,
            key_to_update=key_to_update,
            new_value=new_value,
            dynamic_blocks=dynamic_blocks,
        )
    return config


def update_list_attribute(
    config: list[Any], key_parts: list[str], key_to_update: str, new_value: Any, dynamic_blocks: bool = False
) -> list[Any] | dict[str, Any]:
    """Updates a list attribute in the given config"""

    if not config:
        # happens when we can't correctly evaluate something, because of strange defaults or 'for_each' blocks
        return config

    if len(key_parts) == 1:
        idx = force_int(key_parts[0])
        # Avoid changing the config and cause side effects
        inner_config = pickle_deepcopy(config[0])

        if idx is not None and isinstance(inner_config, list):
            if not inner_config:
                # happens when config = [[]]
                return config

            inner_config[idx] = new_value
            return [inner_config]
    entry_to_update = int(key_parts[0]) if key_parts[0].isnumeric() else -1
    for i, config_value in enumerate(config):
        if entry_to_update == -1:
            config[i] = update_dictionary_attribute(config=config_value, key_to_update=key_to_update, new_value=new_value, dynamic_blocks=dynamic_blocks)
        elif entry_to_update == i:
            config[i] = update_dictionary_attribute(config=config_value, key_to_update=".".join(key_parts[1:]), new_value=new_value, dynamic_blocks=dynamic_blocks)

    return config


def get_vertex_as_tf_module(block: TerraformBlock) -> TFModule:
    block_name = get_sanitized_terraform_resource_id(block.name)
    return TFModule(path=block.path, name=block_name, nested_tf_module=block.source_module_object, foreach_idx=block.for_each_index)



from typing import Any, List

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class FunctionAppMinTLSVersion(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure Function app is using the latest version of TLS encryption"
        id = "CKV_AZURE_145"
        supported_resources = ('Microsoft.Web/sites', 'Microsoft.Web/sites/slots',)
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self) -> str:
        return "properties/siteConfig/minTlsVersion"

    def get_expected_value(self) -> Any:
        return 1.2

    def get_expected_values(self) -> List[Any]:
        return ["1.2", 1.2]


check = FunctionAppMinTLSVersion()



from typing import Dict, List, Any, Union
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_list


class AzureServiceFabricClusterProtectionLevel(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensures that Service Fabric use three levels of protection available"
        id = "CKV_AZURE_125"
        supported_resources = ('Microsoft.ServiceFabric/clusters',)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        properties: Union[List[Any], Dict[str, Any]] = conf.get('properties', {})
        if not isinstance(properties, dict):
            self.evaluated_keys = ['properties']
            return CheckResult.FAILED

        settings_conf = force_list(properties.get('fabricSettings', []))
        if not isinstance(settings_conf, list):
            self.evaluated_keys = ['properties/fabricSettings']
            return CheckResult.FAILED

        for setting in settings_conf:
            if setting and isinstance(setting, dict) and setting.get('name') == 'Security':
                params = setting.get('parameters', [{}])
                if isinstance(params, list) and len(params) > 0 and isinstance(params[0], dict):
                    param = params[0]
                    if param.get('name') == 'ClusterProtectionLevel' and param.get('value') == 'EncryptAndSign':
                        index = settings_conf.index(setting)
                        self.evaluated_keys = [f'fabricSettings/{index}/parameters/name',
                                               f'fabricSettings/{index}/parameters/value']
                        return CheckResult.PASSED
                else:
                    self.evaluated_keys = [f'fabricSettings/{settings_conf.index(setting)}/parameters']
                    return CheckResult.FAILED

        self.evaluated_keys = ['fabricSettings']
        return CheckResult.FAILED


check = AzureServiceFabricClusterProtectionLevel()



from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.enums import CheckCategories


class CloudWatchAlarmsEnabled(BaseResourceNegativeValueCheck):
    def __init__(self):
        """
        NIST.800-53.r5 AU-6(1), NIST.800-53.r5 AU-6(5), NIST.800-53.r5 CA-7,
        NIST.800-53.r5 SI-2, NIST.800-53.r5 SI-4(12)
        CloudWatch alarm actions should be activated
        """
        name = "Ensure that CloudWatch alarm actions are enabled"
        id = "CKV_AWS_319"
        supported_resources = ['aws_cloudwatch_metric_alarm']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "actions_enabled"

    def get_forbidden_values(self):
        return [False]


check = CloudWatchAlarmsEnabled()



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class ZoneUserIDIncludeACL(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure an Include ACL is defined for a Zone when User-ID is enabled"
        id = "CKV_PAN_15"
        supported_resources = ('panos_zone', 'panos_panorama_zone')
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:

        # Report the area of evaluation
        self.evaluated_keys = ['include_acls']

        # Get User-ID status, boolean value
        user_id_enabled = conf.get('enable_user_id')

        # Check if User-ID is enabled in the zone
        if user_id_enabled:

            # Then check if an Include ACL is defined for User-ID
            if 'include_acls' in conf:

                # Get the Include ACL attribute
                acls = conf.get('include_acls')[0]

                # Cycle through each item in the Include ACL list
                for acl in acls:

                    # Check for empty strings
                    if acl.strip() == "":

                        # An empty string is no ACL, which is a fail
                        return CheckResult.FAILED

                # No empty strings found in Include ACL definition, so this is a pass
                return CheckResult.PASSED

            else:
                # No Include ACl for User-ID is a fail
                return CheckResult.FAILED

        # If User-ID is not enabled for the zone, the Include ACL check is not needed
        else:
            return CheckResult.PASSED


check = ZoneUserIDIncludeACL()



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from checkov.common.images.graph.image_referencer_manager import GraphImageReferencerManager
from checkov.common.typing import LibraryGraph
from checkov.kustomize.image_referencer.provider.kustomize import KustomizeProvider

if TYPE_CHECKING:
    from checkov.common.images.image_referencer import Image


class KustomizeImageReferencerManager(GraphImageReferencerManager):

    def __init__(self, graph_connector: LibraryGraph, report_mutator_data: dict[str, dict[str, Any]]):
        super().__init__(graph_connector)
        self.report_mutator_data = report_mutator_data

    def extract_images_from_resources(self) -> list[Image]:
        kustomize_provider = KustomizeProvider(graph_connector=self.graph_connector, report_mutator_data=self.report_mutator_data)
        images = kustomize_provider.extract_images_from_resources()

        return images



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class SagemakerDataQualityJobDefinitionTrafficEncryption(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Amazon Sagemaker Data Quality Job encrypts all communications between instances used for monitoring jobs"
        id = "CKV_AWS_369"
        supported_resources = ['aws_sagemaker_data_quality_job_definition']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "network_config/[0]/enable_inter_container_traffic_encryption"


check = SagemakerDataQualityJobDefinitionTrafficEncryption()



from typing import Any, List

from checkov.cloudformation.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.enums import CheckCategories


class MSKClusterNodesArePrivate(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure MSK nodes are private"
        id = "CKV_AWS_291"
        supported_resources = ['AWS::MSK::Cluster']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'Properties/BrokerNodeGroupInfo/ConnectivityInfo/PublicAccess/Type'

    def get_forbidden_values(self) -> List[Any]:
        return ["SERVICE_PROVIDED_EIPS"]


check = MSKClusterNodesArePrivate()



from __future__ import annotations

from typing import Any, TYPE_CHECKING

from checkov.bitbucket_pipelines.registry import registry
from checkov.common.bridgecrew.check_type import CheckType
from checkov.yaml_doc.runner import Runner as YamlRunner

if TYPE_CHECKING:
    from checkov.common.checks.base_check_registry import BaseCheckRegistry


class Runner(YamlRunner):
    check_type = CheckType.BITBUCKET_PIPELINES  # noqa: CCE003  # a static attribute

    def __init__(self) -> None:
        super().__init__()

    def require_external_checks(self) -> bool:
        return False

    def import_registry(self) -> BaseCheckRegistry:
        return registry

    @staticmethod
    def is_workflow_file(file_path: str) -> bool:
        """
        :return: True if the file mentioned is named bitbucket-pipelines.yml. Otherwise: False
        """
        return file_path.endswith(("bitbucket-pipelines.yml", "bitbucket-pipelines.yaml"))

    @staticmethod
    def _parse_file(
        f: str, file_content: str | None = None
    ) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
        if Runner.is_workflow_file(f):
            return YamlRunner._parse_file(f)

        return None



from typing import Any

from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories


class LambdaInVPC(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that AWS Lambda function is configured inside a VPC"
        id = "CKV_AWS_117"
        supported_resources = ("AWS::Lambda::Function", "AWS::Serverless::Function")
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "Properties/VpcConfig"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = LambdaInVPC()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class NeptuneClusterBackupRetention(BaseResourceCheck):
    def __init__(self):
        description = "Ensure that Neptune DB cluster has automated backups enabled with adequate retention"
        id = "CKV_AWS_361"
        supported_resources = ['aws_neptune_cluster']
        categories = [CheckCategories.BACKUP_AND_RECOVERY]
        super().__init__(name=description, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if conf.get('backup_retention_period', [1])[0] >= 7:
            return CheckResult.PASSED

        return CheckResult.FAILED

    def get_evaluated_keys(self):
        return 'backup_retention_period'


check = NeptuneClusterBackupRetention()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories
from typing import Any
from checkov.common.models.consts import ANY_VALUE


class EFSAccessUserIdentity(BaseResourceValueCheck):
    def __init__(self):
        """
        NIST.800-53.r5 AC-6(2)
        EFS access points should enforce a user identity
        """
        name = "EFS access points should enforce a user identity"
        id = "CKV_AWS_330"
        supported_resources = ['aws_efs_access_point']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "posix_user/[0]/gid"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = EFSAccessUserIdentity()



from typing import Optional, Any, Dict

from checkov.common.graph.checks_infra.enums import Operators
from .contains_attribute_solver import ContainsAttributeSolver


class NotContainsAttributeSolver(ContainsAttributeSolver):
    operator = Operators.NOT_CONTAINS  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        return not super()._get_operation(vertex, attribute)

    def should_check_all_condition(self) -> bool:
        return True
